@@ -1,42 +1,6 @@
-/* data.c
- *
- * $Log: data.c,v $
- * Revision 1.10  2006/09/19 10:20:39  quozl
- * ut06 full screen, det circle, quit on motd, add icon, add desktop file
- *
- * Revision 1.9  2006/05/22 13:13:39  quozl
- * change defaults
- *
- * Revision 1.8  2001/08/21 20:52:15  siegl
- *
- * mouse wheel support
- *
- * Revision 1.7  2001/04/28 04:03:56  quozl
- * change -U to also adopt a local port number for TCP mode.
- * 		-- Benjamin `Quisar' Lerman  <quisar@quisar.ambre.net>
- *
- * Revision 1.6  2000/12/28 12:36:38  siegl
- * CVS info
- *
- * Revision 1.5  2000/05/19 14:24:52  jeffno
- * Improvements to playback.
- * - Can jump to any point in recording.
- * - Can lock on to cloaked players.
- * - Tactical/galactic repaint when paused.
- * - Can lock on to different players when recording paused.
- *
- * Revision 1.4  1999/08/05 16:46:32  siegl
- * remove several defines (BRMH, RABBITEARS, NEWDASHBOARD2)
- *
- * Revision 1.3  1999/07/24 19:23:43  siegl
- * New default portSwap for UDP_PORTSWAP feature
- *
- * Revision 1.2  1999/06/11 16:14:17  siegl
- * cambot replay patches
- *
- * Revision 1.1.1.1  1998/11/01 17:24:09  siegl
- * COW 3.0 initial revision
- * */
+/*
+ * data.c
+ */
 #include "config.h"
 #include "copyright.h"
 
@@ -93,7 +57,7 @@ int     gen_distress = 0;			 /* generic distress/macro *
 						  * 
 						  * * system support */
 int     niftyNewMessages = 1;
-int     oldalert = 0;
+int     oldalert = PFGREEN;
 int     remap[16] =
 {0, 1, 2, 0, 3, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0};
 int     messpend = 0;
@@ -151,19 +115,14 @@ unsigned LONG netaddr = 0;			 /* for blessing */
 #endif
 
 int     msgBeep = 1;				 /* ATM - msg beep */
-int     scanmapped = 0;				 /* ATM - scanners */
 
 int     logmess = 0;
 int     continuetractor = 1;
 int     tcounter = 2;
 int     showlocal = 2;
 int     showgalactic = 2;
-
-#ifdef HAVE_XPM
 int     pixMissing = 0;
 int     pixFlags = 0;
-
-#endif
 char   *title = NULL;
 char   *shipnos = "0123456789abcdefghijklmnopqrstuvwxyz";
 
@@ -411,7 +370,7 @@ char    login[PSEUDOSIZE];
 struct ship shipvals[NUM_TYPES];
 
 /* 10 Aug 96 - Added curt (short) names -SAC */
-struct rank ranks[NUMRANKS] =
+struct rank default_ranks[DEFAULT_NUMRANKS] =
 {
   {0.0, 0.0, 0.0, "Ensign", "Esgn"},
   {2.0, 1.0, 0.0, "Lieutenant", "Lt "},
@@ -422,11 +381,13 @@ struct rank ranks[NUMRANKS] =
   {25.0, 6.0, 0.0, "Commodore", "Cdor"},
   {30.0, 7.0, 0.0, "Rear Adm.", "RAdm"},
   {40.0, 8.0, 0.0, "Admiral", "Admr"}};
+struct rank *ranks = NULL;
+int nranks = 0;
 
 W_Window messagew, w, mapw, statwin, baseWin = 0, infow, iconWin, tstatw,
         war, warnw, helpWin, teamWin[4], qwin, messwa, messwt, messwi,
         messwk, playerw, planetw, rankw, optionWin = 0, reviewWin;
-W_Window scanw, scanwin, udpWin, phaserwin;
+W_Window scanw, udpWin, phaserwin;
 
 #ifdef SHORT_PACKETS
 W_Window spWin = NULL;
@@ -551,7 +512,7 @@ char   *distlite[NUM_DIST] =
 {
   NULL, 
   "/c/l/0/|%T%c Taking %L|",
-  "/|Ogg %T%c|",
+  "/|Ogg %p|",
   "/|Bomb %L|",
   "/|Control %L|",
   "/|Save %L|",
@@ -700,8 +661,8 @@ char   *bugURL = "http://bugzilla.us.netrek.org/cow?version=%s&pl=%i&arch=%s";
 #endif
 
 #ifdef SOUND
-int     sound_init = 1;
-int     sound_toggle = 0;
+int    sound_init = 1;
+int    sound_toggle = 0;
 char   *sounddir = NULL;
 W_Window soundWin = NULL;
 
@@ -730,12 +691,8 @@ int     F_self_8flags2 = 0;
 int     F_ship_cap = 0;
 int     F_sp_generic_32 = 1;
 char    A_sp_generic_32 = 0;
-
-#ifdef HAVE_XPM
 int     F_agri_pix = 1;
 
-#endif
-
 #ifdef WARP_DEAD
 int     F_dead_warp = 0;
 
@@ -751,3 +708,7 @@ int F_show_all_tractors = 1;
 int fastGuest = 0;
 int identityBlind = 0;
 int versionHide = 0;
+int F_sp_rank = 0;
+int F_sp_ltd = 0;
+int F_tips = 1;
+char *program = NULL;
