@@ -1,20 +1,10 @@
-
 /* map.c
  *
  * Functions to maintain the galactic map.  This file is a merger
  * of code from redraw.c, which was too big before, and planets.c,
  * which was too small.
  *
- * $Log: map.c,v $
- * Revision 1.3  1999/08/05 16:46:32  siegl
- * remove several defines (BRMH, RABBITEARS, NEWDASHBOARD2)
- *
- * Revision 1.2  1999/01/31 16:38:17  siegl
- * Hockey rink background XPM on galactic map in hockey mode.
- *
- * Revision 1.1.1.1  1998/11/01 17:24:10  siegl
- * COW 3.0 initial revision
- * */
+ */
 
 #include "config.h"
 #include "copyright2.h"
@@ -25,12 +15,12 @@
 #include "defs.h"
 #include "struct.h"
 #include "data.h"
+
 #include "map.h"
 
 /* debugging feature, show rectangular redraw regions on galactic */
 #undef DEBUG_SHOW_REGIONS
 
-#ifdef HAVE_XPM
 extern void *S_mPlanet(int);
 extern void *S_mArmy(int);
 extern void *S_mRepair(int);
@@ -38,8 +28,6 @@ extern void *S_mFuel(int);
 extern void *S_mOwner(int);
 extern int W_DrawSprite(void *, int, int, int);
 
-#endif
-
 /*
  *  Local Constants:
  *  
@@ -296,12 +284,8 @@ static void DrawPlanets()
   register struct planet *l;
   register int dx, dy;
   char    ch;
-
-#ifdef HAVE_XPM
   void   *sprite;
 
-#endif
-
   for (l = planets + MAXPLANETS - 1; l >= planets; --l)
     {
       if (!(l->pl_flags & PLREDRAW))
@@ -376,7 +360,6 @@ static void DrawPlanets()
 	{
 #endif
 
-#ifdef HAVE_XPM
 	  sprite = S_mPlanet(l->pl_no);
 
 	  if (sprite == NULL)
@@ -394,7 +377,6 @@ static void DrawPlanets()
 	    }
 
 	  else
-#endif /* HAVE_XPM */
 
 	    W_OverlayBitmap(dx - (mplanet_width / 2), dy - (mplanet_height / 2),
 			    planetmBitmap(l), planetColor(l));
@@ -567,14 +549,12 @@ void
 	  return;
 	}
 
-#ifdef HAVE_XPM
 #ifdef HOCKEY_LINES
       if (hockey_s_lines)
         W_GalacticBgd(HOCKEY_PIX);
       else
 #endif
         W_GalacticBgd(MAP_PIX);
-#endif
 
       W_ClearWindow(mapw);
       clearlock = 0;
