@@ -16,7 +16,7 @@
 #include "defs.h"
 #include "struct.h"
 #include "data.h"
-#include "playerlist.h"
+
 #include "bitmaps.h"
 #include "moobitmaps.h"
 #include "rabbitbitmaps.h"
@@ -31,22 +31,61 @@
 
 #include "oldbitmaps.h"
 #include "packets.h"
-#include "spopt.h"
+
+#include "colors.h"
+#include "cowmain.h"
+#include "death.h"
+#include "defaults.h"
+#include "lagmeter.h"
+#include "input.h"
+#include "interface.h"
+#include "macrowin.h"
+#include "map.h"
+#include "netstatopt.h"
 #include "option.h"
-#include "newwin.h"
+#include "pingstats.h"
+#include "playerlist.h"
 #include "redraw.h"
+#include "smessage.h"
+#include "socket.h"
+#include "spopt.h"
 #include "udpopt.h"
+#include "war.h"
+#include "warning.h"
+
+#include "newwin.h"
 
 extern char cbugs[];
 
-static int line = 0;
-int     MaxMotdLine = 0;
+/* line number of motd to display at top of window */
+static int motd_offset = 0;
+
+/* number of motd lines received since clear */
+int motd_last = 0;
+
+/* number of times we have received a motd clear */
+int motd_clears = 0;
+
+/* whether a refresh of the motd should be done on the next SP_MASK */
+int motd_refresh_needed = 0;
 
 /* if a motd line from the server is this, the client will junk all motd *
  * data it currently has.  New data may be received */
-#define MOTDCLEARLINE  "\033\030CLEAR_MOTD\000"
+#define MOTDCLEARLINE   "\033\030CLEAR_MOTD\000"
 
-#define SIZEOF(a)       (sizeof (a) / sizeof (*(a)))
+/* number of lines of motd to display in window at a time */
+#define MOTD_PAGE_SIZE  28
+
+/* forward linked list of received motd lines */
+struct motd_line
+{
+  struct motd_line *next;
+  char *data;
+  int bold;
+};
+
+/* pointer to first item in list */
+static struct motd_line *motd_lines = NULL;
 
 #define BOXSIDE         (TWINSIDE / 5)
 #define TILESIDE        16
@@ -82,17 +121,24 @@ static char press_bits[] =
 
 /* Event Handlers. */
 extern void drawIcon(void), redrawTstats(void), planetlist(void);
-extern void ranklist(void), fillhelp(void), fillmacro(void);
+extern void ranklist(void), fillhelp(void);
 extern void redrawLMeter(void), redrawPStats(void), redrawStats(void);
 
 extern void nsaction(W_Event * data);
 extern void optionaction(W_Event * data);
-extern void waraction(W_Event * data);
+extern int checkMapped(char *name);
+extern int checkMappedPref(char *name, int preferred);
+extern void pastebuffer(void);
 
-/* Other function declarations */
-extern int smessage(char ichar);
-
-static int teamRequest(int team, int ship);
+/* internal function declarations */
+static void savebitmaps(void);
+static void showValues(struct motd_line *data);
+static void ClearMotd(void);
+static void getResources(char *prog);
+static void getTiles(void);
+static void redrawTeam(W_Window win, int teamNo, int *lastnum);
+static void redrawQuit(void);
+static void showTimeLeft(time_t time, time_t max);
 
 static void
         handleMessageWindowKeyDown(W_Event * event)
@@ -133,36 +179,49 @@ pid_t newwin_fork()
   return pid;
 }
 
+/*! @brief Minimal windowing interface initialisation for non-play. */
+void newwinmeta(char *hostmon, char *progname)
+{
+  newwin_last_hostmon = hostmon;
+  newwin_last_progname = progname;
+
+  W_Initialize(hostmon);
+  getResources(progname);
+}
+
+/*! @brief Windowing interface initialisation for play. */
 void newwin(char *hostmon, char *progname)
 {
-  int main_width = TWINSIDE + GWINSIDE + BORDER;
-  int main_height = GWINSIDE + 2 * BORDER + PLISTSIZE;
+  int need_width = TWINSIDE + GWINSIDE + BORDER * 3;
+  int need_height = GWINSIDE + 4 * BORDER + PLISTSIZE;
+  int main_width = MAX(need_width, 1024);
+  int main_height = MAX(need_height, 768);
+  int ow = (main_width - need_width) / 2; /* offset width */
+
+  if (ow < 0) ow = 0;
 
   newwin_last_hostmon = hostmon;
   newwin_last_progname = progname;
 
   W_Initialize(hostmon);
+  W_FullScreenInitialise();
 
-  if (booleanDefault("FullScreen", 1)) {
-    main_width = MAX(main_width, 1024);
-    main_height = MAX(main_height, 768);
-  }
-
-  baseWin = W_MakeWindow("netrek", 0, YOFF, main_width,
-		   main_height, NULL, BORDER, gColor);
+  baseWin = W_MakeWindow("netrek", 0, YOFF, main_width, main_height, NULL,
+                         BORDER, gColor);
 
   iconWin = W_MakeWindow("netrek_icon", 0, 0, icon_width, icon_height, NULL,
-			 BORDER, gColor);
+                         BORDER, gColor);
   W_SetWindowExposeHandler(iconWin, drawIcon);
 
   W_SetIconWindow(baseWin, iconWin);
-  w = W_MakeWindow("local", 0, 0, TWINSIDE, TWINSIDE, baseWin,
+  w = W_MakeWindow("local", ow, 0, TWINSIDE, TWINSIDE, baseWin,
 		   BORDER, foreColor);
 
-  mapw = W_MakeWindow("map", TWINSIDE+BORDER, 0, GWINSIDE, GWINSIDE, baseWin,
-		      BORDER, foreColor);
+  mapw = W_MakeWindow("map", TWINSIDE+BORDER+ow, 0,
+		      GWINSIDE, GWINSIDE, baseWin, BORDER, foreColor);
+  W_GetPixmaps(w, mapw);
 
-  tstatw = W_MakeWindow("tstat", 0, TWINSIDE+BORDER, TWINSIDE, STATSIZE,
+  tstatw = W_MakeWindow("tstat", ow, TWINSIDE+BORDER, TWINSIDE, STATSIZE,
 			baseWin, BORDER, foreColor);
 
 #ifdef nodef					 /* 01/18/95 No messages for
@@ -172,11 +231,11 @@ void newwin(char *hostmon, char *progname)
 
   W_SetWindowExposeHandler(tstatw, redrawTstats);
 
-  warnw = W_MakeWindow("warn", TWINSIDE+BORDER, GWINSIDE+BORDER, GWINSIDE, MESSAGESIZE,
+  warnw = W_MakeWindow("warn", ow+TWINSIDE+BORDER, GWINSIDE+BORDER, GWINSIDE, MESSAGESIZE,
 		       baseWin, BORDER, foreColor);
   W_SetWindowKeyDownHandler(warnw, handleMessageWindowKeyDown);
 
-  messagew = W_MakeWindow("message", TWINSIDE+BORDER, GWINSIDE+BORDER+MESSAGESIZE,
+  messagew = W_MakeWindow("message", ow+TWINSIDE+BORDER, GWINSIDE+BORDER+MESSAGESIZE,
 			  GWINSIDE, MESSAGESIZE, baseWin, BORDER, foreColor);
   W_SetWindowKeyDownHandler(messagew, handleMessageWindowKeyDown);
   W_SetWindowButtonHandler(messagew, handleMessageWindowButton);
@@ -184,7 +243,7 @@ void newwin(char *hostmon, char *progname)
   planetw = W_MakeTextWindow("planet", 10, 10, 57, MAXPLANETS + 3, w, 2);
   W_SetWindowExposeHandler(planetw, planetlist);
 
-  rankw = W_MakeTextWindow("rank", 50, 300, 65, NUMRANKS + 9, w, 2);
+  rankw = W_MakeTextWindow("rank", 48, 300, 65, nranks + 9, w, 2);
   W_SetWindowExposeHandler(rankw, ranklist);
 
 #ifdef SMALL_SCREEN
@@ -192,7 +251,7 @@ void newwin(char *hostmon, char *progname)
 			     YOFF + GWINSIDE + 2 * BORDER + 2 * MESSAGESIZE,
 			     PlistMaxWidth(), MAXPLAYER + 3, baseWin, 2);
 #else
-  playerw = W_MakeTextWindow("player", 0,
+  playerw = W_MakeTextWindow("player", ow,
 			     YOFF + TWINSIDE + BORDER + STATSIZE + BORDER,
 			     PlistMaxWidth(), MAXPLAYER + 3, baseWin, 2);
 #endif
@@ -206,39 +265,39 @@ void newwin(char *hostmon, char *progname)
 #ifdef SMALL_SCREEN
   /* note that wk and phaswerwin are drawn under wi, and wa under wt-- there
    * * just isn't ROOM for them all */
-  messwt = W_MakeScrollingWindow("review_team", 0,
+  messwt = W_MakeScrollingWindow("review_team", ow,
 				 YOFF + TWINSIDE + BORDER + STATSIZE,
 				 80, 1, baseWin, 1);
-  messwi = W_MakeScrollingWindow("review_your", 0,
+  messwi = W_MakeScrollingWindow("review_your", ow,
 				 YOFF + TWINSIDE + MESSAGESIZE + STATSIZE,
 				 80, 1, baseWin, 1);
-  messwa = W_MakeScrollingWindow("review_all", 0,
+  messwa = W_MakeScrollingWindow("review_all", ow,
 			       YOFF + TWINSIDE + 2 * MESSAGESIZE + STATSIZE,
 				 80, 1, baseWin, 1);
-  messwk = W_MakeScrollingWindow("review_kill", 0,
+  messwk = W_MakeScrollingWindow("review_kill", ow,
 			       YOFF + TWINSIDE + 3 * MESSAGESIZE + STATSIZE,
 				 35, 1, baseWin, 1);
-  phaserwin = W_MakeScrollingWindow("review_phaser", 25 + 35 * W_Textwidth,
+  phaserwin = W_MakeScrollingWindow("review_phaser", ow + 25 + 35 * W_Textwidth,
 			       YOFF + TWINSIDE + 3 * MESSAGESIZE + STATSIZE,
 				    40, 1, baseWin, 1);
-  reviewWin = W_MakeScrollingWindow("review", 0,
+  reviewWin = W_MakeScrollingWindow("review", ow,
 	       YOFF + TWINSIDE + BORDER + STATSIZE, 80, 2, baseWin, BORDER);
 #else
-  messwa = W_MakeScrollingWindow("review_all", TWINSIDE + BORDER,
+  messwa = W_MakeScrollingWindow("review_all", ow + TWINSIDE + BORDER,
    YOFF + GWINSIDE + 3 * BORDER + 2 * MESSAGESIZE, 80, 10, baseWin, BORDER);
-  messwt = W_MakeScrollingWindow("review_team", TWINSIDE + BORDER,
+  messwt = W_MakeScrollingWindow("review_team", ow + TWINSIDE + BORDER,
      YOFF + GWINSIDE + 4 * BORDER + 2 * MESSAGESIZE + 10 * W_Textheight + 8,
 				 80, 5, baseWin, BORDER);
-  messwi = W_MakeScrollingWindow("review_your", TWINSIDE + BORDER,
+  messwi = W_MakeScrollingWindow("review_your", ow + TWINSIDE + BORDER,
     YOFF + GWINSIDE + 5 * BORDER + 2 * MESSAGESIZE + 15 * W_Textheight + 16,
 				 80, 4, baseWin, BORDER);
-  messwk = W_MakeScrollingWindow("review_kill", TWINSIDE + BORDER,
+  messwk = W_MakeScrollingWindow("review_kill", ow + TWINSIDE + BORDER,
     YOFF + GWINSIDE + 6 * BORDER + 2 * MESSAGESIZE + 19 * W_Textheight + 24,
 				 80, 6, baseWin, BORDER);
-  phaserwin = W_MakeScrollingWindow("review_phaser", TWINSIDE + BORDER,
+  phaserwin = W_MakeScrollingWindow("review_phaser", ow + TWINSIDE + BORDER,
     YOFF + GWINSIDE + 3 * BORDER + 2 * MESSAGESIZE + 15 * W_Textheight + 16,
 				    80, 4, baseWin, BORDER);
-  reviewWin = W_MakeScrollingWindow("review", TWINSIDE + BORDER + 1,
+  reviewWin = W_MakeScrollingWindow("review", ow + TWINSIDE + BORDER + 1,
 	      YOFF + GWINSIDE + BORDER + STATSIZE, 81, 21, baseWin, BORDER-1);
 #endif
 
@@ -311,7 +370,6 @@ void newwin(char *hostmon, char *progname)
 
   W_SetWindowExposeHandler(statwin, redrawStats);
 
-  scanwin = W_MakeWindow("scanner", 422, 13, 160, 120, baseWin, 5, foreColor);
   W_DefineArrowCursor(netstatWin);
   W_DefineArrowCursor(lMeter);
   W_DefineTrekCursor(baseWin);
@@ -333,7 +391,6 @@ void newwin(char *hostmon, char *progname)
   W_DefineTextCursor(messagew);
   W_DefineTrekCursor(tstatw);
   W_DefineWarningCursor(qwin);
-  W_DefineTrekCursor(scanwin);
   W_DefineArrowCursor(udpWin);
 
 #ifdef SHORT_PACKETS
@@ -349,7 +406,7 @@ void newwin(char *hostmon, char *progname)
 #define WARWIDTH 20
 #define WARBORDER 2
 
-  war = W_MakeMenu("war", TWINSIDE + 10, -BORDER + 10, WARWIDTH, 6, baseWin,
+  war = W_MakeMenu("war", ow + TWINSIDE + 10, -BORDER + 10, WARWIDTH, 6, baseWin,
 		   WARBORDER);
   W_SetWindowButtonHandler(war, waraction);
 
@@ -393,7 +450,7 @@ void mapAll(void)
   if (checkMapped("review_kill") && !identityBlind)
     W_MapWindow(messwk);
   if (checkMapped("netstat"))
-    nswindow(netstatWin);
+    nswindow();
   if (checkMapped("lagMeter"))
     W_MapWindow(lMeter);
   if (checkMapped("pingStats"))
@@ -417,7 +474,7 @@ void mapAll(void)
 
 }
 
-savebitmaps(void)
+static void savebitmaps(void)
 {
   register int i;
 
@@ -813,9 +870,9 @@ void entrywindow(int *team, int *s_type)
   int     chosen_ship = -1;
   int     request_sent = 0;
   time_t  request_time;
-  time_t  quittime = 60;
+  time_t  quittime = 120;
   time_t  lasttime = -1;
-  time_t  spareTime = 1000;
+  time_t  elapsed;
   static int previous_team = -1;
   static int previous_ship = -1;
 
@@ -845,7 +902,7 @@ void entrywindow(int *team, int *s_type)
   *team = -1;
   startTime = time(NULL);
   if (me->p_whydead != KWINNER && me->p_whydead != KGENOCIDE)
-    showMotd(w, line);
+    showMotd(w, motd_offset);
 
   run_clock(startTime);
   updatedeath();
@@ -866,7 +923,6 @@ void entrywindow(int *team, int *s_type)
   pickOk = -1;
   while (1) {
     while (!W_EventsPending()) {
-      time_t  elapsed;
       fd_set  rfds;
       struct timeval tv;
       int retval;
@@ -887,9 +943,8 @@ void entrywindow(int *team, int *s_type)
         request_sent++;
         request_time = time(NULL);
       }
-      
+
 #ifndef HAVE_WIN32
-      W_FullScreen(baseWin);
       tv.tv_sec = 1; /* rate at which quit clock is updated */
 #else
       tv.tv_sec = 0;
@@ -971,13 +1026,6 @@ void entrywindow(int *team, int *s_type)
       lastOkayMask = okayMask;
     } /* while graphics events are not pending */
 
-    if (time(NULL) - startTime <= spareTime) {
-      spareTime -= time(NULL) - startTime;
-      startTime = time(NULL);
-    } else {
-      startTime += spareTime;
-      spareTime = 0;
-    }
     W_NextEvent(&event);
     switch ((int) event.type) {
     case W_EV_KEY:
@@ -1058,40 +1106,40 @@ void entrywindow(int *team, int *s_type)
           }
           break;
         case 'f':                        /* Scroll motd forward */
-          line = line + 28;
-          if (line > MaxMotdLine) {
-            line = line - 28;
+          motd_offset = motd_offset + MOTD_PAGE_SIZE;
+          if (motd_offset > motd_last) {
+            motd_offset = motd_offset - MOTD_PAGE_SIZE;
             break;
           }
           W_ClearWindow(w);
-          showMotd(w, line);
+          showMotd(w, motd_offset);
           break;
         case 'b':                        /* Scroll motd backward */
-          if (line == 0)
+          if (motd_offset == 0)
             break;
-          line = line - 28;
-          if (line < 0)
-            line = 0;
+          motd_offset = motd_offset - MOTD_PAGE_SIZE;
+          if (motd_offset < 0)
+            motd_offset = 0;
           W_ClearWindow(w);
-          showMotd(w, line);
+          showMotd(w, motd_offset);
           break;
         case 'F':                        /* Scroll motd forward */
-          line = line + 4;
-          if (line > MaxMotdLine) {
-            line = line - 4;
+          motd_offset = motd_offset + 4;
+          if (motd_offset > motd_last) {
+            motd_offset = motd_offset - 4;
             break;
           }
           W_ClearWindow(w);
-          showMotd(w, line);
+          showMotd(w, motd_offset);
           break;
         case 'B':                        /* Scroll motd backward */
-          if (line == 0)
+          if (motd_offset == 0)
             break;
-          line = line - 4;
-          if (line < 0)
-            line = 0;
+          motd_offset = motd_offset - 4;
+          if (motd_offset < 0)
+            motd_offset = 0;
           W_ClearWindow(w);
-          showMotd(w, line);
+          showMotd(w, motd_offset);
           break;
         }
       }
@@ -1120,13 +1168,17 @@ void entrywindow(int *team, int *s_type)
       if (event.Window == qwin) {
         run_clock(lasttime);
         redrawQuit();
+        showTimeLeft(elapsed, quittime);
       } else if (event.Window == tstatw) {
         redrawTstats();
       } else if (event.Window == iconWin) {
         drawIcon();
       } else if (event.Window == w) {
         run_clock(lasttime);
-        showMotd(w, line);
+        showMotd(w, motd_offset);
+      } else if (event.Window == mapw) {
+        redrawall = 1;
+        map();
       } else if (event.Window == helpWin) {
         fillhelp();
 #ifdef NBT
@@ -1151,7 +1203,7 @@ void entrywindow(int *team, int *s_type)
   }
 }
 
-static int deadTeam(int owner) /* unused */
+int deadTeam(int owner) /* unused */
 
 /* The team is dead if it has no planets and cannot coup it's home planet */
 {
@@ -1172,119 +1224,99 @@ static int deadTeam(int owner) /* unused */
   return (1);
 }
 
-checkBold(char *line)
-/* Determine if that line should be highlighted on sign-on screen */
-/* Which is done when it is the players own score being displayed */
-
+/*! @brief Check if a line should be highlighted on motd
+    @details The line is checked at a particular known position for
+    the player character name.  Intended for use in highlighting your
+    score in a list of scores.
+    @returns zero for no, one for yes.
+*/
+static int checkBold(char *line)
 {
   char   *s, *t;
   int     i;
   int     end = 0;
 
+  if (me == NULL)
+    return 0;
+
   if (strlen(line) < 60)
-    return (0);
+    return 0;
+
   s = line + 4;
   t = me->p_name;
 
-  if (me == NULL)
-    return (0);
-
-  for (i = 0; i < 16; i++)
-    {
-      if (!end)
-	{
-	  if (*t == '\0')
-	    end = 1;
-	  else if (*t != *s)
-	    return (0);
-	}
-      if (end)
-	{
-	  if (*s != ' ')
-	    return (0);
-	}
-      s++;
-      t++;
+  for (i = 0;i < 16;i++) {
+    if (!end) {
+      if (*t == '\0')
+        end = 1;
+      else if (*t != *s)
+        return 0;
     }
-  return (1);
+    if (end) {
+      if (*s != ' ')
+        return 0;
+    }
+    s++;
+    t++;
+  }
+  return 1;
 }
 
-struct list
+static void show_motd_heading(W_Window w, char *text, int line, int colour)
 {
-  char    bold;
-  struct list *next;
-  char   *data;
-};
-static struct list *motddata = NULL;		 /* pointer to first bit of * 
+  int length = strlen(text);
+  int center = TWINSIDE / 2 - (length * W_Textwidth) / 2;
+  W_WriteText(w, center, line * W_Textheight, colour,
+	      text, length, W_RegularFont);
+}
 
-						  * 
-						  * 
-						  * * motddata */
-static int first = 1;
-
-void showMotd(W_Window motdwin, int atline)
+/*! @brief Redraw the message of the day (MOTD)
+*/
+void showMotd(W_Window w, int atline)
 {
   int     i, length, top, center;
-  struct list *data;
+  struct motd_line *data;
   int     count;
   char    buf[128];
 
   sprintf(buf, "---  %s  ---", (char *) query_cowid());
   length = strlen(buf);
   center = TWINSIDE / 2 - (length * W_Textwidth) / 2;
-  W_WriteText(motdwin, center, W_Textheight, W_Cyan,
-	      buf, length, W_BoldFont);
+  W_WriteText(w, center, W_Textheight, W_Cyan, buf, length, W_BoldFont);
 
-  sprintf(buf, cbugs);
-  length = strlen(buf);
-  center = TWINSIDE / 2 - (length * W_Textwidth) / 2;
-  W_WriteText(motdwin, center, 3 * W_Textheight, W_Cyan,
-	      buf, length, W_RegularFont);
+  show_motd_heading(w, cbugs, 3, W_Cyan);
 
-  sprintf(buf, "this is the team selection window");
-  length = strlen(buf);
-  center = TWINSIDE / 2 - (length * W_Textwidth) / 2;
-  W_WriteText(motdwin, center, 5 * W_Textheight, W_Grey,
-	      buf, length, W_RegularFont);
+  if (me == NULL) {
+    show_motd_heading(w, "while you are in the queue", 5, W_Grey);
+    show_motd_heading(w, "do some reading or play somewhere else", 6, W_Grey);
+  } else {
+    show_motd_heading(w, "this is the team selection window", 5, W_Grey);
+    show_motd_heading(w, "press enter to join the default team", 6, W_Grey);
+  }
 
-  sprintf(buf, "press enter to join the default team");
-  length = strlen(buf);
-  center = TWINSIDE / 2 - (length * W_Textwidth) / 2;
-  W_WriteText(motdwin, center, 6 * W_Textheight, W_Grey,
-	      buf, length, W_RegularFont);
-
-  sprintf(buf, "server message of the day follows below");
-  length = strlen(buf);
-  center = TWINSIDE / 2 - (length * W_Textwidth) / 2;
-  W_WriteText(motdwin, center, 9 * W_Textheight, W_Grey,
-	      buf, length, W_RegularFont);
+  show_motd_heading(w, "server message follows below", 9, W_Grey);
 
   top = 11;
 
-  if (first)
-    {
-      first = 0;
-      data = motddata;
-      while (data != NULL)
-	{
-	  data->bold = checkBold(data->data);
-	  data = data->next;
-	}
-    }
+  data = motd_lines;
+  while (data != NULL) {
+    if (data->bold == -1)
+      data->bold = checkBold(data->data);
+    data = data->next;
+  }
 
-  data = motddata;
+  data = motd_lines;
   for (i = 0; i < atline; i++)
     {
       if (data == NULL)
 	{
 	  atline = 0;
-	  data = motddata;
+	  data = motd_lines;
 	  break;
 	}
       data = data->next;
     }
-  count = 28;					 /* Magical # of lines to * * 
-						  * display */
+  count = MOTD_PAGE_SIZE;
   for (i = top; i < 50; i++)
     {
       if (data == NULL)
@@ -1293,12 +1325,12 @@ void showMotd(W_Window motdwin, int atline)
 	break;
       if (data->bold)
 	{
-	  W_WriteText(motdwin, 20, i * W_Textheight, textColor, data->data,
+	  W_WriteText(w, 20, i * W_Textheight, textColor, data->data,
 		      strlen(data->data), W_BoldFont);
 	}
       else
 	{
-	  W_WriteText(motdwin, 20, i * W_Textheight, textColor, data->data,
+	  W_WriteText(w, 20, i * W_Textheight, textColor, data->data,
 		      strlen(data->data), W_RegularFont);
 	}
       data = data->next;
@@ -1307,14 +1339,15 @@ void showMotd(W_Window motdwin, int atline)
 	break;
     }
 
-  showValues(data);
+  if (!motd_clears) showValues(data);
 }
 
-/* ATM: show the current values of the .sysdef parameters. */
-showValues(struct list *data)
+/*! @brief Show the current values of the .sysdef parameters.
+*/
+static void showValues(struct motd_line *data)
 {
   int     i;
-  static char *msg = "OPTIONS SET WHEN YOU STARTED WERE:";
+  static char *msg = "Server options set:";
 
   /* try to find the start of the info */
   while (1)
@@ -1329,83 +1362,107 @@ showValues(struct list *data)
 
   W_WriteText(mapw, 20, 14 * W_Textheight, textColor, msg,
 	      strlen(msg), W_RegularFont);
-  for (i = 16; i < 50; i += 2)
+  for (i = 16; i < 50; i++)
     {
+      int y = i * W_Textheight * 15 / 10;
+
       if (data == NULL)
 	break;
       if (data->data[0] == '+')			 /* quick boldface hack */
-	W_WriteText(mapw, 20, i * W_Textheight, textColor, data->data + 1,
+	W_WriteText(mapw, 20, y, textColor, data->data + 1,
 		    strlen(data->data) - 1, W_BoldFont);
       else
-	W_WriteText(mapw, 20, i * W_Textheight, textColor, data->data,
+	W_WriteText(mapw, 20, y, textColor, data->data,
 		    strlen(data->data), W_RegularFont);
       data = data->next;
     }
 }
 
+/*! @brief Store a newly received SP_MOTD text line for display.
+*/
 void newMotdLine(char *line)
 {
-  static struct list **temp = &motddata;
-  static int statmode = 0;			 /* ATM */
+  static struct motd_line *old = NULL; /* previous item allocated */
+  static int statmode = 0;
+  struct motd_line *new;
 
-  if (!statmode && !strcmp(line, STATUS_TOKEN))
+  if (strcmp(line, MOTDCLEARLINE) == 0) {
+    W_ClearWindow(w);
+    ClearMotd();
+    statmode = 0;
+    return;
+  }
+
+  if (!strcmp(line, STATUS_TOKEN))
     statmode = 1;
+
+  new = (struct motd_line *) malloc(sizeof(struct motd_line));
+  if (new == NULL)
+    return;
+
   if (!statmode)
-    MaxMotdLine++;				 /* ATM - don't show on left */
-  (*temp) = (struct list *) malloc(sizeof(struct list));
+    motd_last++;
 
-  if ((*temp) == NULL)
-    {						 /* malloc error checking --
-						  * * * 10/30/92 EM */
-      printf("Warning:  Couldn't malloc space for a new motd line!");
-      return;
-    }
-  /* Motd clearing code */
-  if (strcmp(line, MOTDCLEARLINE) == 0)
-    {
-      ClearMotd();
-      return;
-    }
-
-  (*temp)->next = NULL;
-  (*temp)->data = malloc(strlen(line) + 1);
-  strcpy((*temp)->data, line);
-  temp = &((*temp)->next);
+  /* add new line to tail of list */
+  new->next = NULL;
+  new->bold = -1;
+  new->data = strdup(line);
+  if (motd_lines == NULL) {
+    motd_lines = new;
+  } else {
+    old->next = new;
+  }
+  old = new;
+  motd_refresh_needed++;
 }
 
-/* Free the current motdData */
-ClearMotd(void)
+/*! @brief Refresh the displayed MOTD
+    @details If a refresh is known to be needed, it is done.  Called
+    on receipt of an SP_MASK following a server's stream of SP_MOTD,
+    as SP_MASK is not sent until SP_MOTD is completed and ready to be
+    displayed.
+*/
+void motd_refresh()
 {
-  struct list *temp, *temp2;
+  if (motd_refresh_needed) {
+    showMotd(w, motd_offset);
+    motd_refresh_needed = 0;
+  }
+}
 
-  temp = motddata;				 /* start of motd information 
-						  * 
-						  */
-  while (temp != NULL)
-    {
-      temp2 = temp;
-      temp = temp->next;
-      free(temp2->data);
-      free(temp2);
-    }
+/*! @brief Refresh the displayed MOTD
+*/
+static void ClearMotd(void)
+{
+  struct motd_line *next, *this;
 
-  first = 1;					 /* so that it'll check bold
-						  * * * next time around */
+  next = motd_lines; /* start of motd information */
+  while (next != NULL) {
+    this = next;
+    next = next->next;
+    free(this->data);
+    free(this);
+  }
+
+  motd_lines = NULL;
+  motd_offset = 0;
+  motd_last = 0;
+  motd_clears++;
 }
 
 /* ARGSUSED */
-getResources(char *prog)
+static void getResources(char *prog)
 {
   getColorDefs();
   getTiles();
 }
 
-getTiles(void)
+static void getTiles(void)
 {
   stipple = W_StoreBitmap(stipple_width, stipple_height, stipple_bits, w);
 }
 
-redrawTeam(W_Window win, int teamNo, int *lastnum)
+static void redrawTeam(W_Window win, int teamNo, int *lastnum)
 {
   char    buf[BUFSIZ];
   static char *teams[] =
@@ -1425,7 +1482,7 @@ redrawTeam(W_Window win, int teamNo, int *lastnum)
   *lastnum = num;
 }
 
-redrawQuit(void)
+static void redrawQuit(void)
 {
   char *msg = "Quit";
   int tx = W_WindowWidth(qwin) / 2 - W_Textwidth * strlen(msg) / 2;
@@ -1445,7 +1502,7 @@ void    drawIcon(void)
 
 #define XPI             3.141592654
 
-showTimeLeft(time_t time, time_t max)
+static void showTimeLeft(time_t time, time_t max)
 {
   char    buf[BUFSIZ], *cp;
   int     cx, cy, ex, ey, tx, ty;
@@ -1467,7 +1524,7 @@ showTimeLeft(time_t time, time_t max)
   ey = cy - clock_height * cos(2 * XPI * time / max) / 2;
   W_MakeLine(qwin, cx, cy, ex, ey, foreColor);
 
-  sprintf(buf, "%d", max - time);
+  sprintf(buf, "%d", (int) (max - time));
   tx = cx - W_Textwidth * strlen(buf) / 2;
   ty = cy - W_Textheight / 2;
   W_WriteText(qwin, tx, ty, textColor, buf, strlen(buf), W_RegularFont);
