@@ -20,60 +20,63 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <errno.h>
+
 #include "Wlib.h"
 #include "defs.h"
 #include "struct.h"
 #include "data.h"
-#include "defaults.h"
+#include "version.h"
+#include "patchlevel.h"
 
 #include "badversion.h"
-#include "parsemeta.h"
+#include "defaults.h"
 #include "newwin.h"
 
+#include "parsemeta.h"
+
 #ifdef WIN32 /* socket garbage in case the client is not running on NT */
 #define read(f,b,l) recv(f,b,l,0)
 #define write(f,b,l) send(f,b,l,0)
 #define close(s) closesocket(s)
 #endif
 
-
 /* Constants */
 
 #define BUF     6144
 #define LINE    100             /* Width of a meta-server line           */
-#define MAXMETABYTES 2048	/* maximum metaserver UDP packet size    */
+#define MAXMETABYTES 2048       /* maximum metaserver UDP packet size    */
 static int msock = -1;          /* the socket to talk to the metaservers */
 static int sent = 0;            /* number of solicitations sent          */
 static int seen = 0;            /* number of replies seen                */
 static int verbose = 0;         /* whether to talk a lot about it all    */
-static int type;		/* type of connection requested          */
-static time_t last;		/* time of last refresh                  */
+static time_t last;             /* time of last refresh                  */
 
 /* Local Types */
 
 struct servers {
-  char    address[LINE];	/* host name or ip address of server	*/
+  char    address[LINE];        /* host name or ip address of server    */
   int     port;
-  int     age;			/* age in seconds as received		*/
-  time_t  when;			/* date time this record received	*/
-  int     refresh;		/* data needs redisplaying		*/
-  int     lifetime;		/* remaining cache life of entry        */
+  int     age;                  /* age in seconds as received           */
+  time_t  when;                 /* date time this record received       */
+  int     refresh;              /* data needs redisplaying              */
+  int     lifetime;             /* remaining cache life of entry        */
   int     players;
   int     status;
   char    typeflag;
   char    comment[LINE];
-  pid_t   pid;			/* our last known child playing here    */
-  int     exitstatus;		/* exit status of last known child here */
-  int     observer;		/* set if child is an observer          */
+  pid_t   pid;                  /* our last known child playing here    */
+  int     exitstatus;           /* exit status of last known child here */
+  int     observer;             /* set if child is an observer          */
 };
 
-struct servers *serverlist = NULL;	/* The record for each server.  */
-static int num_servers = 0;		/* The number of servers.       */
-static int chosen = -1;			/* Arrow key chosen server.     */
-static int metaHeight = 0;		/* The number of list lines.	*/
-static char *metaWindowName;		/* The window's name.           */
+struct servers *serverlist = NULL;      /* The record for each server.  */
+static int num_servers = 0;             /* The number of servers.       */
+static int chosen = -1;                 /* Arrow key chosen server.     */
+static int metaHeight = 0;              /* The number of list lines.    */
+static char *metaWindowName;            /* The window's name.           */
 static int statusLevel;
-static W_Window metaWin, metaHelpWin = NULL;
+static W_Window metaWin, metaList, metaHelpWin = NULL;
+void *logo;
 
 /* button offsets from end of list */
 #define B_ADD 4
@@ -89,10 +92,10 @@ static W_Window metaWin, metaHelpWin = NULL;
 /* The status strings:  The order of the strings up until statusNull is
  * important because the meta-client will display all the strings up to a
  * particular point.
- * 
+ *
  * The strings after statusNull are internal status types and are formatted
  * separatly from the other strings.
- * 
+ *
  * The string corresponding to "statusNull" is assigned to thoes servers which
  * have "statusNobody" or earlier strings in old, cached, meta-server data. */
 
@@ -114,18 +117,54 @@ char *metahelp_message[] =
   {
     "Netrek Server List - Help",
     "",
+    "This is a list of Netrek servers, from the Metaserver, or on your",
+    "local network.  You can do this:",
+    "",
     "  -  click on a server to join the game there,",
     "",
     "  -  middle-click on a server to immediately join as guest,",
     "",
     "  -  right-click on a server to observe the game there,",
     "",
-    "  -  click to refresh the list of servers,",
+    "  -  click refresh to reload the list of servers,",
     "",
-    "The information shown comes from the Netrek Metaserver,",
-    "or from any Netrek servers nearby on your local network.",
+    "-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --",
     "",
-    "For more information on Netrek, visit www.netrek.org/beginner",
+    "Tips for new Netrek players:",
+    "",
+    "  1.  when you enter the game, press the number 4 to start moving,",
+    "  other numbers are slower or faster,",
+    "",
+    "  2.  you are the white ship in the centre of the left-hand screen,",
+    "",
+    "  3.  use right-click on the mouse to steer toward a point in space,",
+    "  you will find it easier to turn at lower speeds,",
+    "",
+    "  4.  use left-click on the mouse to fire a torpedo, they travel over",
+    "  time so you have to point ahead of your target, they hurt enemy",
+    "  ships but usually pass right through your team,",
+    "",
+    "  5.  use middle-click on your mouse to fire a phaser, they are",
+    "  instant so point at your target, but only work well close up,",
+    "",
+    "  6.  the aim of the game is to capture planets, killing enemies is",
+    "  only a means to an end, and dying is a good thing, because you get a",
+    "  new ship,",
+    "",
+    "  7.  but if someone kills you, they can begin to capture planets, so",
+    "  it is best to not die in vain,",
+    "",
+    "  8.  people in your team will try to communicate so that you can all",
+    "  do things together, because when you cooperate better than the other",
+    "  team, you win.",
+    "",
+    "  9.  use the 'h' key to see a list of commands for your ship.",
+    "",
+    "Observing is a good way to learn.  When you join as an observer, you",
+    "won't see much until you use 'l' (lower-case L) to lock on to a",
+    "player.  Choose a good player and watch what they do.",
+    "",
+    "For more information on Netrek, visit http://netrek.org/beginner",
     NULL
   };
 
@@ -178,167 +217,13 @@ static void toggle_help()
   }
 }
 
-static int open_port(char *host, int port, int verbose)
-/* The connection to the metaserver is by Andy McFadden. This calls the
- * metaserver and parses the output into something useful */
-{
-  struct sockaddr_in addr;
-  struct hostent *hp;
-  int     sock;
-
-  /* Connect to the metaserver */
-  /* get numeric form */
-  if ((addr.sin_addr.s_addr = inet_addr(host)) == -1) {
-    if ((hp = gethostbyname(host)) == NULL) {
-      if (verbose) fprintf(stderr, "unknown host '%s'\n", host);
-      return (-1);
-    } else {
-      addr.sin_addr.s_addr = *(LONG *) hp->h_addr;
-    }
-  }
-  addr.sin_family = AF_INET;
-  addr.sin_port = htons(port);
-  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-    if (verbose) perror("socket");
-    return (-1);
-  }
-  if (connect(sock, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-    if (verbose) perror("connect");
-    close(sock);
-    return (-1);
-  }
-  return (sock);
-}
-
-
-static void parseInput(char *in, FILE * out)
-/* Read the information we need from the meta-server. */
-{
-  char    line[LINE + 1], *numstr, *point, **statStr;
-  struct servers *slist;
-  int     rtn, max_servers;
-  int     count;
-
-#ifdef DEBUG
-   printf("In parseInput\n");
-#endif   
-   
-  /* Create some space to hold the entries that will be read.  More space can
-   * be added later */
-
-  serverlist = (struct servers *) malloc(sizeof(struct servers) * 10);
-
-  max_servers = 10;
-  num_servers = 0;
-
-  /* Add the default server */
-
-  if (serverName) {
-    strcpy(serverlist[num_servers].address, serverName);
-    serverlist[num_servers].port = xtrekPort;
-    serverlist[num_servers].status = statusDefault;
-    serverlist[num_servers].players = 0;
-    serverlist[num_servers].typeflag = ' ';
-    num_servers++;
-  }
-
-  while (1) {
-    /* Read a line */
-    point = line;
-    count = LINE + 1;
-    
-    do {
-      if (!(--count)) {
-	fputs("Warning: Line from meta server was too long!!!\n", stderr);
-	++point;                           /* Pretend we read a '\n' */
-	break;
-      }
-      
-      rtn = *in++;
-      if (!rtn)                              /* use a zero to mark end of buffer */
-	return;
-      
-      *(point++) = rtn;
-    }
-    while (rtn != EOF && rtn != '\n');
-    
-    *(--point) = '\0';
-    
-    if (out != NULL) {                           /* Save a copy of the stuff
-                                                  * we read */
-      fputs(line, out);
-      putc('\n', out);
-    }
-
-      /* Find somewhere to put the information that is just about to be
-       * parsed */
-
-    if (num_servers >= max_servers) {
-      max_servers += 5;
-      size_t size = sizeof(struct servers) * max_servers;
-      serverlist = (struct servers *) realloc(serverlist, size);
-    }
-    
-    slist = serverlist + num_servers;
-
-    /* Is this a line we want? */
-
-    if (sscanf(line, "-h %s -p %d %d %*d",
-	       slist->address, &(slist->port),
-	       &(slist->age)) != 3) {
-      continue;
-    }
-    
-    /* Find the status of the server, eg "Not responding". */
-    
-    for (statStr = statusStrings + statusLevel
-           ; statStr >= statusStrings
-           ; --statStr) {
-      if ((numstr = strstr(line, *statStr)) != NULL) {
-	(slist->status) = statStr - statusStrings;
-	(slist->players) = 0;
-	sscanf(numstr, "%*[^0123456789] %d", &(slist->players));
-	break;
-      }
-    }
-    
-    if (statStr < statusStrings)               /* No status was allocated */
-      continue;
-    
-    
-    /* Read the flags */
-    
-    slist->typeflag = *(point - 1);
-    
-    strcpy(slist->comment, "");
-
-    slist->pid = -1;
-    slist->exitstatus = 0;
-    slist->observer = 0;
-
-    /* Don't list Paradise Servers  */
-    
-    if (slist->typeflag != 'P') {
-      
-#ifdef DEBUG 
-      printf("HOST:%-30s PORT:%-6d %12s %-5d %d %c\n",
-	     serverlist[num_servers].address,
-	     serverlist[num_servers].port,
-	     statusStrings[serverlist[num_servers].status],
-	     serverlist[num_servers].players,
-	     serverlist[num_servers].typeflag);
-#endif
-      
-      ++num_servers;
-    }
-  }
-}
-
 static int ReadMetasSend()
 {
-  char *metaservers;		/* our copy of the metaserver host names */
-  char *token;			/* current metaserver host name          */
-  struct sockaddr_in address;	/* the address of the metaservers	 */
+  char *metaservers;            /* our copy of the metaserver host names */
+  char *token;                  /* current metaserver host name          */
+  struct sockaddr_in address;   /* the address of the metaservers        */
+  static char *req;             /* the request packet for the metaserver */
+  static int reqlen;            /* the length of the request packet      */
 
   last = time(NULL);
 
@@ -356,7 +241,7 @@ static int ReadMetasSend()
   if (msock < 0) {
     msock = socket(AF_INET, SOCK_DGRAM, 0);
     if (msock < 0) { perror("ReadMetasSend: socket"); return 0; }
-    
+
     /* bind the socket to any address */
     address.sin_addr.s_addr = INADDR_ANY;
     address.sin_family      = AF_INET;
@@ -366,17 +251,22 @@ static int ReadMetasSend()
       close(msock);
       return 0;
     }
+    req = malloc(80);
+    snprintf(req, 80, "?version=netrek-client-cow-%s.%d",
+             mvers, PATCHLEVEL);
+    reqlen = strlen(req);
   }
 
   /* send request to a multicast metaserver on local area network */
   address.sin_family = AF_INET;
   address.sin_port = htons(metaport);
   address.sin_addr.s_addr = inet_addr("224.0.0.1");
-  if (verbose) fprintf(stderr, 
-		       "requesting player list from nearby servers on %s\n",
-		       inet_ntoa(address.sin_addr));
-  if (sendto(msock, "?", 1, 0, (struct sockaddr *)&address,
-	     sizeof(address)) < 0) {
+  if (verbose) fprintf(stderr,
+                       "requesting player list from nearby servers on %s\n",
+                       inet_ntoa(address.sin_addr));
+
+  if (sendto(msock, req, reqlen, 0, (struct sockaddr *)&address,
+             sizeof(address)) < 0) {
     perror("ReadMetasSend: sendto");
   } else {
     sent++;
@@ -397,40 +287,40 @@ static int ReadMetasSend()
     /* attempt numeric translation first */
     if ((address.sin_addr.s_addr = inet_addr(token)) == -1) {
       struct hostent *hp;
-        
+
       /* then translation by name */
       if ((hp = gethostbyname(token)) == NULL) {
         /* if it didn't work, return failure and warning */
         fprintf(stderr,
-	  "cannot resolve host %s, check for DNS problems?\n",
-	  token);
+          "cannot resolve host %s, check for DNS problems?\n",
+          token);
       } else {
         int i;
 
         /* resolution worked, send a query to every metaserver listed */
         for(i=0;;i++) {
 
-	  /* check for end of list of addresses */
-	  if (hp->h_addr_list[i] == NULL) break;
-	  address.sin_addr.s_addr = *(long *) hp->h_addr_list[i];
-	  if (verbose) fprintf(stderr,
-		"requesting player list from metaserver %s at %s\n",
-		token, inet_ntoa(address.sin_addr));
-	  if (sendto(msock, "?", 1, 0, (struct sockaddr *)&address,
-		sizeof(address)) < 0) {
-	    perror("ReadMetasSend: sendto");
-	  } else {
-	    sent++;
-	  }
+          /* check for end of list of addresses */
+          if (hp->h_addr_list[i] == NULL) break;
+          address.sin_addr.s_addr = *(long *) hp->h_addr_list[i];
+          if (verbose) fprintf(stderr,
+                "requesting player list from metaserver %s at %s\n",
+                token, inet_ntoa(address.sin_addr));
+          if (sendto(msock, req, reqlen, 0, (struct sockaddr *)&address,
+                     sizeof(address)) < 0) {
+            perror("ReadMetasSend: sendto");
+          } else {
+            sent++;
+          }
         }
       }
     } else {
       /* call to inet_addr() worked, host name is in IP address form */
-      if (verbose) fprintf(stderr, 
-			   "requesting player list from metaserver %s\n",
-			   inet_ntoa(address.sin_addr));
-      if (sendto(msock, "?", 1, 0, (struct sockaddr *)&address,
-	sizeof(address)) < 0) {
+      if (verbose) fprintf(stderr,
+                           "requesting player list from metaserver %s\n",
+                           inet_ntoa(address.sin_addr));
+      if (sendto(msock, req, reqlen, 0, (struct sockaddr *)&address,
+                 sizeof(address)) < 0) {
         perror("ReadMetasSend: sendto");
       } else {
         sent++;
@@ -441,7 +331,7 @@ static int ReadMetasSend()
     token = strtok(NULL,",");
   } /* while (token != NULL) */
 
-  metaWindowName = "Netrek Server List (UDP)";
+  metaWindowName = "Netrek Server List";
   return sent;
 }
 
@@ -486,10 +376,10 @@ static void version_r(struct sockaddr_in *address) {
   if (servers < 0) return;
 
   if (verbose) fprintf(stderr,
-		       "metaserver at %s responded with %d server%s\n",
-		       inet_ntoa(address->sin_addr),
-		       servers,
-		       servers == 1 ? "" : "s" );
+                       "metaserver at %s responded with %d server%s\n",
+                       inet_ntoa(address->sin_addr),
+                       servers,
+                       servers == 1 ? "" : "s" );
 
   if (servers == 0) return;
 
@@ -498,17 +388,17 @@ static void version_r(struct sockaddr_in *address) {
     struct servers *sp = NULL;
     char *host, type;
     int port, status, age, players, queue, throwaway;
-      
+
     throwaway = 0;
 
-    host = strtok(NULL,",");		/* hostname */
+    host = strtok(NULL,",");            /* hostname */
     if (host == NULL) continue;
 
-    p = strtok(NULL,",");		/* port */
+    p = strtok(NULL,",");               /* port */
     if (p == NULL) continue;
     port = atoi(p);
 
-    p = strtok(NULL,",");		/* status */
+    p = strtok(NULL,",");               /* status */
     if (p == NULL) continue;
     status = atoi(p);
 
@@ -517,22 +407,22 @@ static void version_r(struct sockaddr_in *address) {
       throwaway++;
     /* the sp->why_dead workaround */
     if (status == 6)
-      if ((status - 3) <= statusLevel) 
-	throwaway--;
+      if ((status - 3) <= statusLevel)
+        throwaway--;
 
-    p = strtok(NULL,",");		/* age (of data in seconds) */
+    p = strtok(NULL,",");               /* age (of data in seconds) */
     if (p == NULL) continue;
     age = atoi(p);
 
-    p = strtok(NULL,",");		/* players */
+    p = strtok(NULL,",");               /* players */
     if (p == NULL) continue;
     players = atoi(p);
 
-    p = strtok(NULL,",");		/* queue size */
+    p = strtok(NULL,",");               /* queue size */
     if (p == NULL) continue;
     queue = atoi(p);
 
-    p = strtok(NULL,"\n");		/* server type */
+    p = strtok(NULL,"\n");              /* server type */
     if (p == NULL) continue;
     type = p[0];
 
@@ -548,15 +438,15 @@ static void version_r(struct sockaddr_in *address) {
     /* if it was found, check age */
     if (sp != NULL) {
       if ((now-age) < (sp->when-sp->age)) {
-	sp->age = now - (sp->when-sp->age);
-	sp->when = now;
-	sp->refresh = 1;
-	sp->lifetime = 20;
-	continue;
+        sp->age = now - (sp->when-sp->age);
+        sp->when = now;
+        sp->refresh = 1;
+        sp->lifetime = 20;
+        continue;
       } else {
-	sp->age = age;
-	sp->when = now;
-	sp->lifetime = 20;
+        sp->age = age;
+        sp->when = now;
+        sp->lifetime = 20;
       }
     } else {
       /* not found, store it at the end of the list */
@@ -598,9 +488,9 @@ static void version_r(struct sockaddr_in *address) {
       sp->status = statusTout;
       sp->players = 0;
       break;
-    case SS_NOCONN:			/* no connection */
-    case SS_WORKING:		/* metaserver should not return this */
-    case SS_INIT:			/* nor this */
+    case SS_NOCONN:                     /* no connection */
+    case SS_WORKING:            /* metaserver should not return this */
+    case SS_INIT:                       /* nor this */
     default:
       sp->status = statusNoConnect;
       sp->players = 0;
@@ -626,35 +516,35 @@ static void version_s(struct sockaddr_in *address)
 
   if (verbose) fprintf(stderr, "server at %s responded\n", host);
 
-  p = strtok(NULL,",");	/* server type */
+  p = strtok(NULL,","); /* server type */
   if (p == NULL) return;
   char type = p[0];
-  
+
   /* ignore paradise servers */
   if (type == 'P') return;
-  
-  p = strtok(NULL,",");		/* comment */
+
+  p = strtok(NULL,",");         /* comment */
   if (p == NULL) return;
   char *comment = strdup(p);
   if (strlen(comment) > LINE) comment[LINE] = '\0';
 
-  p = strtok(NULL,",");		/* number of ports */
+  p = strtok(NULL,",");         /* number of ports */
   if (p == NULL) return;
-  int ports = atoi(p);		/* not currently used */
+  // int ports = atoi(p);               /* not currently used */
 
   // TODO: accept more than one port reply
-  
-  p = strtok(NULL,",");		/* port */
+
+  p = strtok(NULL,",");         /* port */
   if (p == NULL) return;
   int port = atoi(p);
-  
-  p = strtok(NULL,",");		/* players */
+
+  p = strtok(NULL,",");         /* players */
   if (p == NULL) return;
   int players = atoi(p);
 
-  p = strtok(NULL,",");		/* queue size */
+  p = strtok(NULL,",");         /* queue size */
   if (p == NULL) return;
-  int queue = atoi(p);		/* not currently used */
+  // int queue = atoi(p);               /* not currently used */
 
   /* find in current server list? */
   struct servers *sp = server_find(host, port);
@@ -674,7 +564,11 @@ static void version_s(struct sockaddr_in *address)
   sp->refresh = 1;
   sp->lifetime = 20;
   sp->players = players;
-  sp->status = statusOpen;
+  if (type == 'u' && players == 0) {
+    sp->status = statusNobody;
+  } else {
+    sp->status = statusOpen;
+  }
   sp->typeflag = type;
   strncpy(sp->comment, comment, LINE);
   sp->pid = -1;
@@ -685,12 +579,12 @@ static void version_s(struct sockaddr_in *address)
 
 static int ReadMetasRecv(int x)
 {
-  struct sockaddr_in address;	/* the address of the metaservers	 */
-  socklen_t length;		/* length of the address		 */
-  int bytes;			/* number of bytes received from meta'   */
-  fd_set readfds;		/* the file descriptor set for select()	 */
-  struct timeval timeout;	/* timeout for select() call		 */
-  char packet[MAXMETABYTES];	/* buffer for packet returned by meta'	 */
+  struct sockaddr_in address;   /* the address of the metaservers        */
+  socklen_t length;             /* length of the address                 */
+  int bytes;                    /* number of bytes received from meta'   */
+  fd_set readfds;               /* the file descriptor set for select()  */
+  struct timeval timeout;       /* timeout for select() call             */
+  char packet[MAXMETABYTES];    /* buffer for packet returned by meta'   */
   char *p;
 
   /* now await and process replies */
@@ -766,7 +660,7 @@ static void SaveMetasCache()
         tmpFileName[len-1] = 'R';
       else
         tmpFileName[len-1] = 'T';
-      
+
       cache = fopen(tmpFileName, "w");
 
       if (cache == NULL)
@@ -799,9 +693,9 @@ static void SaveMetasCache()
 #else
       unlink(cacheName);
 #endif
-#endif    
+#endif
       if (rename(tmpFileName, cacheName) == -1)
-	perror("Could not rename new cache file");
+        perror("Could not rename new cache file");
     }
 
 }
@@ -824,28 +718,27 @@ static void LoadMetasCache()
   findfile(cacheName, cacheFileName);
 
   cache = fopen(cacheFileName, "r");
-  if (cache == NULL) 
-    { 
-      num_servers = 0; 
-      return; 
+  if (cache == NULL)
+    {
+      num_servers = 0;
+      return;
     }
- 
+
   /* ignore the cache if user changed statusLevel */
   fread(&i, sizeof(i), 1, cache);
   if (i != statusLevel)
     {
       num_servers = 0;
       fclose(cache);
-      return; 
+      return;
     }
- 
 
   /* read the server list into memory from the file */
   fread(&num_servers, sizeof(num_servers), 1, cache);
   serverlist = (struct servers *) malloc(sizeof(struct servers)*num_servers);
   fread(serverlist, sizeof(struct servers), num_servers, cache);
   fclose(cache);
-  
+
   /* hunt and kill old server lines from cache */
   for(i=0;i<num_servers;i++)
     {
@@ -859,199 +752,21 @@ static void LoadMetasCache()
 
       /* delete this entry by moving the ones above down */
       for(j=i;j<num_servers-1;j++)
-	{
-	  memcpy(&serverlist[j],&serverlist[j+1],sizeof(struct servers));
-	}
-      
+        {
+          memcpy(&serverlist[j],&serverlist[j+1],sizeof(struct servers));
+        }
+
       /* adjust the current entry pointer, limit, and resize the memory */
       i--;
       num_servers--;
-      serverlist = 
-	(struct servers *) realloc(serverlist, 
-				   sizeof(struct servers) * (num_servers));
+      serverlist =
+        (struct servers *) realloc(serverlist,
+                                   sizeof(struct servers) * (num_servers));
     }
 }
 
 
-static int ReadFromMeta()
-/* Read from the meta-server.  Return TRUE on success and FALSE on failure. */
-{
-  FILE   *out;
-  char   *cacheName;
-  char    cacheFileName[PATH_MAX];
-  char    tmpFileName[PATH_MAX];
-  char   *sockbuf, *buf;
-  int     bufleft = BUF - 1;
-  int     len;
-  int     sock;
-
-  if ((getdefault("metaserver")) != NULL)
-    metaserver = getdefault("metaserver");
-
-  metaport = intDefault("metaport", metaport);
-
-  if ((sock = open_port(metaserver, metaport, 1)) <= 0)
-    {
-      fprintf(stderr, "Cannot connect to MetaServer (%s , %d)\n",
-              metaserver, metaport);
-      return 0;
-    }
-
-  /* Allocate a buffer and read until full */
-  buf = sockbuf = (char *)malloc(BUF);
-  while (bufleft > 0 && (len = read(sock, buf, bufleft)) > 0)
-    {
-      bufleft-=len;
-      buf += len;
-#ifdef DEBUG    
-      printf("Read %d bytes from Metaserver\n", len);
-#endif
-    }
-  close (sock);
-  *buf = 0;                   /* End of buffer marker */
-
-  if (len<0)
-    {
-      perror ("read");
-      free(sockbuf);
-      return 0;
-    }
-
-  cacheName = getdefault("metaCache");
-  if (cacheName && !findfile(cacheName, cacheFileName))
-   strcpy(cacheFileName, cacheName);        /* overwrite existing file if possible */
-
-  if (cacheName)
-    {
-      len = strlen(cacheFileName);
-      strcpy(tmpFileName, cacheFileName);
-
-      /* Create a temporary file with roughly the same name */
-      
-      if ((cacheFileName[len - 1] == 'T') || (cacheFileName[len - 1] == 't'))
-        tmpFileName[len-1] = 'R';
-      else
-        tmpFileName[len-1] = 'T';
-      
-      out = fopen(tmpFileName, "w");
-
-      if (out == NULL)
-        {
-          fprintf(stderr,
-                   "Cannot write to the metaCache temporary file `%s'.\n",
-              tmpFileName);
-          fprintf(stderr, "Meta-server read will not be cached.\n");
-        }
-    }
-  else
-    {
-      out = NULL;
-    }
-
-  parseInput(sockbuf, out);
-
-  if (out != NULL)
-  {
-    fclose(out);
-
-#ifdef WIN32
-   /* Can't rename file to existing name under NT */
-#ifdef _MSC_VER
-   _unlink(cacheName);
-#else
-   unlink(cacheName);
-#endif
-#endif    
-    if (rename(tmpFileName, cacheName) == -1)
-      perror("Could not write to cache file");
-   }
-
-  free(sockbuf);
-  metaWindowName = "Netrek Server List (TCP)";
-
-  return 1;
-}
-
-
-static int ReadFromCache()
-/* Read from the cache.  Return TRUE on success and FALSE on failure. */
-{
-  FILE   *in;
-  char   *cacheName;
-  struct  servers *slist;
-  char   *sockbuf, *buf;
-  int     bufleft = BUF - 1;
-  int     len;
-  char    cacheFileName[PATH_MAX];
-  
-  cacheName = getdefault("metaCache");
-
-  if (!cacheName)
-    {
-      fprintf(stderr,
-              "You must define the .xtrekrc variable `metaCache' in\n");
-      fprintf(stderr,
-              "order to use the `show known servers' option.\n");
-      return 0;
-    }
-  else
-     if (!findfile(cacheName, cacheFileName) || !(in = fopen(cacheFileName, "r")) )
-       {
-         fprintf(stderr,
-                 "The metaCache file `%s' is empty or not accessable.\n",
-                 cacheName);
-         return 0;
-       }
-
-
-  /* Allocate a buffer and read until full. Why do we
-     go through this silly stuff? Because sockets are
-     not file handles on all verions of Windows */
-  buf = sockbuf = (char *)malloc(BUF);
-
-  while (bufleft >0 && ((len = fread(buf, 1, bufleft, in)) > 0))
-    {
-    bufleft-=len;
-    buf += len;
-#ifdef DEBUG    
-    printf("Read %d bytes from Metaserver cache file\n", len);
-#endif
-    }
-  *buf = 0;                   /* End of buffer marker */
-  fclose (in);
-
-  if (len<0)
-    {
-    perror ("fread");
-    free(sockbuf);
-    return 0;
-    }
-
-  /* Show all servers known to have been reachable */
-
-  if (statusLevel <= statusNobody)
-    statusLevel = statusNobody;
-
-  parseInput(sockbuf, NULL);
-
-
-  /* Don't promise games from old data */
-  for (slist = serverlist + num_servers - 1
-       ; slist >= serverlist
-       ; --slist)
-    {
-      if (slist->status <= statusNobody)
-        slist->status = statusNull;
-    }
-
-  free(sockbuf);
-  metaWindowName = "Known Servers (Cache)";
-
-  return 1;
-}
-
-
-void    parsemeta(int metaType)
+void    parsemeta()
 /* Read and Parse the metaserver information, either from the metaservers
  * by UDP (1), from a single metaserver by TCP (3), or from the cache (2).
  */
@@ -1063,32 +778,9 @@ void    parsemeta(int metaType)
   else if (statusLevel >= statusNull)
     statusLevel = statusNull - 1;
 
-  type = metaType;
-  switch (type)
-    {
-      case 1:
-        ReadMetasSend();
-        LoadMetasCache();
-        metaHeight = 4 + N_OVERHEAD;
-        return;
-        break;
-      case 2:
-	if (ReadFromCache() || ReadFromMeta()) 
-	  {
-	    metaHeight = num_servers + N_OVERHEAD;
-	    return;
-	  }
-	terminate(0);
-	break;
-      case 3:
-	if (ReadFromMeta() || ReadFromCache()) 
-          {
-            metaHeight = num_servers + N_OVERHEAD;
-            return;
-          }
-	terminate(0);
-	break;
-    }
+  ReadMetasSend();
+  LoadMetasCache();
+  metaHeight = 2 + N_OVERHEAD;
 }
 
 
@@ -1101,7 +793,7 @@ static void redraw(int i)
   /* can't say a thing if line is beyond server list */
   if (i >= num_servers) {
     /* but we can at least blank the line shown */
-    if (i < metaHeight-3) W_WriteText(metaWin, 0, i+1, W_White, "", 0, 0);
+    if (i < metaHeight-3) W_WriteText(metaList, 0, i+1, W_White, "", 0, 0);
     return;
   }
 
@@ -1150,33 +842,32 @@ static void redraw(int i)
           strcat(buf, "Unknown ");
           break;
         }
-      
-      if (type == 1)
-	{
-	  int age = sp->age;
-	  char *units;
 
-	  if (age > 86400)
-	    {
-	      age = age / 86400;
-	      units = "d";
-	    }
-	  else if (age > 3600)
-	    {
-	      age = age / 3600;
-	      units = "h";
-	    }
-	  else if (age > 90)
-	    {
-	      age = age / 60;
-	      units = "m";
-	    }
-	  else
-	    {
+        {
+          int age = sp->age;
+          char *units;
+
+          if (age > 86400)
+            {
+              age = age / 86400;
+              units = "d";
+            }
+          else if (age > 3600)
+            {
+              age = age / 3600;
+              units = "h";
+            }
+          else if (age > 90)
+            {
+              age = age / 60;
+              units = "m";
+            }
+          else
+            {
               units = "s";
-	    }
-	  sprintf(buf + strlen(buf), " %4d%s", age, units);
-	}
+            }
+          sprintf(buf + strlen(buf), " %4d%s", age, units);
+        }
     }
 
   strcat(buf, "    ");
@@ -1214,7 +905,7 @@ static void redraw(int i)
   if (sp->status == statusCantConnect) color = W_Red;
   if (sp->pid != -1) color = W_Cyan;
 
-  W_WriteText(metaWin, 0, i+1, color, buf, -1, 0);
+  W_WriteText(metaList, 0, i+1, color, buf, -1, 0);
   sp->refresh = 0;
 }
 
@@ -1233,7 +924,7 @@ static void add_redraw()
   char buf[LINE + 1];
 
   snprintf(buf, LINE, "Add a server: %s_", add_buffer);
-  W_WriteText(metaWin, 0, metaHeight-B_ADD, W_Yellow, buf, -1, 0);
+  W_WriteText(metaList, 0, metaHeight-B_ADD, W_Yellow, buf, -1, 0);
 }
 
 static void add_commit()
@@ -1286,26 +977,30 @@ static int add_key(W_Event *data)
 void    metawindow()
 /* Show the meta server menu window */
 {
-  int     i;
+  int i, height;
   char *header;
+  static int lastHeight = 0;
 
   if (!metaWin) {
-    metaWin = W_MakeMenu("Netrek Server List", 0, 0,
-                         LINE, metaHeight, NULL, 2);
+    height = 250 + metaHeight * (W_Textheight + 8) + 4 * (metaHeight - 1);
+    metaWin = W_MakeWindow("Netrek Server List", 0, 0, 716, height, NULL, 2,
+                           foreColor);
+    W_SetBackgroundImage(metaWin, "Misc/map_back.png");
+    logo = W_ReadImage(metaWin, "netrek-green-white-300px.png");
+    metaList = W_MakeMenu("metalist", 50, 200, LINE, metaHeight, metaWin, 1);
+    lastHeight = metaHeight;
     make_help();
   } else {
-    if (W_WindowHeight(metaWin) != metaHeight) {
-      W_ReinitMenu(metaWin, LINE, metaHeight);
-      W_ResizeMenu(metaWin, LINE, metaHeight);
+    if (metaHeight > lastHeight) {
+      W_ReinitMenu(metaList, LINE, metaHeight);
+      W_ResizeMenu(metaList, LINE, metaHeight);
+      lastHeight = metaHeight;
     }
+    // FIXME: handle metaList growing beyond metaWin
   }
 
-  if (type == 1) {
-    header = "Server                                           Status        Type       Age";
-  } else {
-    header = "Server                                           Status        Type";
-  }
-  W_WriteText(metaWin, 0, 0, W_Cyan, header, -1, 0);
+  header = "Server                                           Status        Type      Age";
+  W_WriteText(metaList, 0, 0, W_Cyan, header, -1, 0);
 
   for (i = 0; i < metaHeight; i++) redraw(i);
 
@@ -1313,35 +1008,38 @@ void    metawindow()
   W_RenameWindow(metaWin, metaWindowName);
 
   /* Add additional options */
-  if (type == 1)
-    W_WriteText(metaWin, 0, metaHeight-B_REFRESH, W_Yellow,
-                "Refresh                                           (r)",
-                -1, 0);
+  W_WriteText(metaList, 0, metaHeight-B_REFRESH, W_Yellow,
+              "Refresh                                           (r)",
+              -1, 0);
   add_redraw();
-  W_WriteText(metaWin, 0, metaHeight-B_HELP, W_Yellow,
-	        "Help                                              (h)",
-	      -1, 0);
-  W_WriteText(metaWin, 0, metaHeight-B_QUIT, W_Yellow,
+  W_WriteText(metaList, 0, metaHeight-B_HELP, W_Yellow,
+                "Help & Tips                                       (h)",
+              -1, 0);
+  W_WriteText(metaList, 0, metaHeight-B_QUIT, W_Yellow,
                 "Quit                                              (q)",
-	      -1, 0);
+              -1, 0);
 
   /* Map window */
+  W_MapWindow(metaList);
   W_MapWindow(metaWin);
 }
 
 
 static void metadone(void)
 {
+  W_UnmapWindow(metaList);
+  W_DropImage(logo);
   W_UnmapWindow(metaWin);
-  if (type == 1) SaveMetasCache();
+  SaveMetasCache();
   free(serverlist);
 }
 
 
 static void refresh()
 {
-  W_WriteText(metaWin, 0, metaHeight-B_REFRESH, W_Red,
+  W_WriteText(metaList, 0, metaHeight-B_REFRESH, W_Red,
               "Refresh (in progress)", -1, 0);
+  W_NextScreenShot(metaWin, 0, 0);
   W_Flush();
   ReadMetasSend();
 }
@@ -1364,8 +1062,10 @@ static void refresh_cyclic()
   }
 
   /* don't do until sufficient time has elapsed */
-  if ((time(NULL) - last) > interval)
+  if ((time(NULL) - last) > interval) {
+    W_NextScreenShot(metaWin, 0, 0);
     ReadMetasSend();
+  }
 }
 
 
@@ -1398,6 +1098,7 @@ static void choose(int way)
 static int chose(int which, unsigned char key)
 {
   struct servers *sp;
+  pid_t pid;
 
   if (which != chosen) {
     int was;
@@ -1420,15 +1121,30 @@ static int chose(int which, unsigned char key)
   sp->pid = -1;
   sp->exitstatus = EXIT_UNKNOWN;
   fprintf(stderr, "you chose server %s port %d\n", serverName, xtrekPort);
-  pid_t pid = newwin_fork();
 
-  if (pid == 0) {
-    /* we are the child */
-    char buf[80];
-    sprintf(buf, "Netrek  @  %s", serverName);
-    W_RenameWindow(baseWin, buf);
-    if (key == W_MBUTTON) fastGuest++;
-    return 1;
+  if ((pid = fork()) == 0) {
+    char *args[16];
+    int argc = 0;
+
+    setpgid(0, 0);
+    args[argc++] = program;
+    args[argc++] = "--server";
+    args[argc++] = serverName;
+    if (xtrekPort != DEFAULT_PORT) {
+      char port[32];
+      sprintf(port, "%d", xtrekPort);
+      args[argc++] = "--port";
+      args[argc++] = strdup(port);
+    }
+
+    if (key == W_MBUTTON)
+      args[argc++] = "--fast-guest";
+
+    args[argc++] = NULL;
+
+    execvp(program, args);
+    perror("execvp");
+    _exit(1);
   }
 
   /* we are the parent, did the fork fail? */
@@ -1451,7 +1167,7 @@ static int button(W_Event *data)
   if ((data->y > 0) && (data->y <= num_servers)) { /* click on server */
     return chose(data->y - 1, data->key);
   }
-  if (data->y == (metaHeight-B_REFRESH) && type == 1) { /* refresh */
+  if (data->y == (metaHeight-B_REFRESH)) { /* refresh */
     refresh();
   } else if (data->y == metaHeight-B_HELP) { /* help */
     toggle_help();
@@ -1537,7 +1253,7 @@ void sigchld(int ignored)
 
 void    metainput(void)
 /* Wait for actions in the meta-server window.
- * 
+ *
  * This is really the meta-server window's own little input() function. It is
  * needed so we don't have to use all the bull in the main input(). Plus to
  * use it I'd have to call mapAll() first and the client would read in the
@@ -1547,32 +1263,37 @@ void    metainput(void)
 
   (void) SIGNAL(SIGCHLD, sigchld);
   while (W_IsMapped(metaWin)) {
-    if (type == 1) {
-      do {
+    while (1) {
+      W_Flush();
+      if (W_EventsPending()) break;
+      if (ReadMetasRecv(W_Socket()) || metareap_needed) {
+        metareap();
+        metaHeight = num_servers + N_OVERHEAD;
+        metawindow();
         W_Flush();
-        if (ReadMetasRecv(W_Socket()) || metareap_needed) {
-          metareap();
-          metaHeight = num_servers + N_OVERHEAD;
-          metawindow();
-          W_Flush();
-        }
-        refresh_cyclic();
-      } while (!W_EventsPending());
+      }
+      refresh_cyclic();
     }
     W_NextEvent(&data);
     switch ((int) data.type) {
     case W_EV_KEY:
-      if (data.Window == metaWin)
+      if (data.Window == metaList || data.Window == metaWin)
         if (key(&data)) return;
       if (data.Window == metaHelpWin) hide_help();
       break;
     case W_EV_BUTTON:
-      if (data.Window == metaWin)
+      if (data.Window == metaList)
         if (button(&data)) return;
       if (data.Window == metaHelpWin) hide_help();
+      if (data.Window == metaWin && data.y < 200)
+        W_NextScreenShot(metaWin, 0, 0);
       break;
     case W_EV_EXPOSE:
       if (data.Window == metaHelpWin) expo_help();
+      if (data.Window == metaWin) {
+        W_DrawScreenShot(metaWin, 0, 0);
+        W_DrawImage(200, 9, logo);
+      }
       break;
     case W_EV_CLOSED:
       if (data.Window == metaWin) {
