@@ -1,46 +1,34 @@
-
-/*
- * $Log: x11sprite.c,v $
- * Revision 1.4  2006/05/22 13:11:58  quozl
- * fix compilation warnings
- *
- * Revision 1.3  1999/06/11 16:14:17  siegl
- * cambot replay patches
- *
- * Revision 1.2  1999/01/31 16:38:17  siegl
- * Hockey rink background XPM on galactic map in hockey mode.
- *
- * Revision 1.1.1.1  1998/11/01 17:24:12  siegl
- * COW 3.0 initial revision
- * */
-
 #include "config.h"
 
-#ifdef HAVE_XPM
 #include <stdio.h>
+#include <unistd.h>
 #include <stdlib.h>
+#include <strings.h>
 #include <sys/param.h>
 
 #include <sys/stat.h>
+#include <fts.h>
 #include <unistd.h>
 
 #include <X11/Xlib.h>
-#include INC_XPM
+#include <Imlib2.h>
 
 #include "Wlib.h"
 #include "defs.h"
 #include "struct.h"
 #include "data.h"
-#include "x11window.h"
-#include "x11sprite.h"
-#include "defaults.h"
 
-#define W_Void2Window(win) ((win) ? ((struct window *) (win)) : (mylocal))
+#include "defaults.h"
+#include "x11window.h"
+
+#include "x11sprite.h"
+
+#define W_Void2Window(win) (((struct window *) (win)))
 #define NoPixmapError 0
 
 struct S_Object
   {
-    W_Window *parent;
+    Drawable drawable;
     GC      gc;
     int     view, nviews, width, height, cloak;
     Pixmap  image;
@@ -87,172 +75,180 @@ extern Visual *W_Visual;
 extern int takeNearest;
 
 extern Display *W_Display;
-struct window *mylocal;
-static GC pixmap_gc;
 
 const char teamnames[NUMTEAM + 1][4] =
 {"Ind", "Fed", "Rom", "Kli", "Ori"};
 
 const char mplanetfiles[NUM_PL_IMGS][12] =
-{"UNKN.xpm",
- "ROCK.xpm",
- "AGRI.xpm",
- "army.xpm",
- "repair.xpm",
- "fuel.xpm",
- "Ind.xpm",
- "Fed.xpm",
- "Rom.xpm",
- "Kli.xpm",
- "Ori.xpm"
+{"UNKN.png",
+ "ROCK.png",
+ "AGRI.png",
+ "army.png",
+ "repair.png",
+ "fuel.png",
+ "Ind.png",
+ "Fed.png",
+ "Rom.png",
+ "Kli.png",
+ "Ori.png"
 };
 const char shipfiles[NUM_TYPES][8] =
-{"SC.xpm",
- "DD.xpm",
- "CA.xpm",
- "BB.xpm",
- "AS.xpm",
- "SB.xpm",
- "GA.xpm",
- "AT.xpm"
+{"SC.png",
+ "DD.png",
+ "CA.png",
+ "BB.png",
+ "AS.png",
+ "SB.png",
+ "GA.png",
+ "AT.png"
 };
 const char torpfiles[2][14] =
-{"torp.xpm", "torp_det.xpm"};
+{"torp.png", "torp_det.png"};
 const char plasmafiles[2][16] =
-{"plasma.xpm", "plasma_det.xpm"};
-const char cloakfile[10] = "cloak.xpm";
+{"plasma.png", "plasma_det.png"};
+const char cloakfile[10] = "cloak.png";
 const char explosionfiles[2][18] =
-{"explosion.xpm", "sbexplosion.xpm"};
+{"explosion.png", "sbexplosion.png"};
 const char bgfiles[NUM_BG_IMGS][16] =
-{"map_back.xpm",
- "local_back.xpm",
- "ghostbust.xpm",
- "genocide.xpm",
- "greet.xpm",
- "hockey.xpm"
+{"map_back.png",
+ "local_back.png",
+ "ghostbust.png",
+ "genocide.png",
+ "greet.png",
+ "hockey.png"
 };
 
-int     ReadFileToSprite(char *filename, struct S_Object *sprite, W_Window * w)
+
+static int ReadFileToSprite(char *filename, struct S_Object *sprite, Drawable drawable)
 {
-  register int k;
-  XpmAttributes attr;
-  int     nviews = 0;
-  struct window *win = W_Void2Window(mylocal);
+  Imlib_Image im;
+  int width, height, nviews;
 
+  if (access(filename, R_OK) != 0) {
+    fprintf(stderr, "image %s is not readable\n", filename);
+    goto fail;
+  }
 
-  attr.visual = W_Visual;
-  attr.colormap = W_Colormap;
-  attr.exactColors = takeNearest;
-  attr.closeness = 40000;
-  attr.valuemask = XpmVisual | XpmColormap | XpmExactColors |
-      XpmReturnExtensions | XpmCloseness;
+  im = imlib_load_image(filename);
+  if (!im) {
+    fprintf(stderr, "image %s failed to load\n", filename);
+    goto fail;
+  }
 
-  if (XpmReadFileToPixmap(W_Display, win->window, filename, &sprite->image,
-			  &sprite->shape, &attr) != XpmSuccess)
-    {
-      if (!(pixMissing & NO_PIXMAPS))
-	fprintf(stderr, "Unable to read file %s.\n", filename);
-      sprite->image = NoPixmapError;
-      sprite->nviews = 1;
-      return (1);
-    }
+  imlib_context_set_display(W_Display);
+  imlib_context_set_visual(W_Visual);
+  imlib_context_set_colormap(W_Colormap);
+  imlib_context_set_image(im);
+  imlib_context_set_drawable(drawable);
+  imlib_render_pixmaps_for_whole_image(&sprite->image, &sprite->shape);
 
-  for (k = 0; k < attr.nextensions; k++)
-    {
-      if (strcmpi(attr.extensions[0].name, "num_views") == 0)
-	nviews = atoi(attr.extensions[0].lines[0]);
-    }
+  width = imlib_image_get_width();
+  height = imlib_image_get_height();
+  nviews = height / width;
+  if (nviews * width != height) {
+    nviews = 1;
+  }
 
-  if (nviews == 0)
-    {
-      int     guess;
-
-      guess = (int) ((attr.height) / (attr.width));
-      if (guess == (attr.height) / (attr.width))
-	{
-	  nviews = guess;
-	}
-      else
-	{
-	  if (!(pixMissing & NO_PIXMAPS))
-	    fprintf(stderr, "NUM_VIEWS not suppied in %s.  Unable to estimate ...\n",
-		    filename);
-	  sprite->image = NoPixmapError;
-	  sprite->nviews = 1;
-	  return (1);
-	}
-    }
-
-  sprite->parent = w;
+  sprite->drawable = drawable;
   sprite->gc = XCreateGC(W_Display, sprite->image, 0, NULL);
+  sprite->view = 0;
   sprite->nviews = nviews;
-  sprite->width = attr.width;
-  sprite->height = (attr.height) / nviews;
+  sprite->width = width;
+  sprite->height = height / nviews;
+  sprite->cloak = 0;
+#ifdef DEBUG
+  fprintf(stderr, "image %s loaded, nv=%d w=%d h=%d (%d)\n", filename,
+          nviews, width, height, sprite->height);
+#endif
+  imlib_free_image_and_decache();
+  return 0;
 
-  return (0);
+ fail:
+  sprite->image = NoPixmapError;
+  sprite->nviews = 1;
+  return 1;
 }
 
-int     ReadFileToTile(char *filename, Pixmap * pix)
+int     ReadFileToTile(char *filename, Pixmap *pix, Drawable drawable)
 {
-  XpmAttributes attr;
-  struct window *win = W_Void2Window(mylocal);
+  Imlib_Image im;
 
-  attr.visual = W_Visual;
-  attr.colormap = W_Colormap;
-  attr.exactColors = takeNearest;
-  attr.closeness = 40000;
-  attr.valuemask = XpmVisual | XpmColormap | XpmExactColors |
-      XpmReturnExtensions | XpmCloseness;
+  if (access(filename, R_OK) != 0) {
+    fprintf(stderr, "image %s is not readable\n", filename);
+    goto fail;
+  }
 
-  if (XpmReadFileToPixmap(W_Display, win->window, filename, pix, NULL, &attr)
-      != XpmSuccess)
-    {
-      if (!(pixMissing & NO_PIXMAPS))
-	fprintf(stderr, "Unable to read file %s.\n", filename);
-      *pix = NoPixmapError;
-      return (1);
-    }
+  im = imlib_load_image(filename);
+  if (!im) {
+    fprintf(stderr, "image %s failed to load\n", filename);
+    goto fail;
+  }
 
-  return (0);
+  imlib_context_set_display(W_Display);
+  imlib_context_set_visual(W_Visual);
+  imlib_context_set_colormap(W_Colormap);
+  imlib_context_set_image(im);
+  imlib_context_set_drawable(drawable);
+  imlib_render_pixmaps_for_whole_image(pix, NULL);
+#ifdef DEBUG
+  fprintf(stderr, "tile %s loaded, w=%d h=%d\n", filename,
+          imlib_image_get_width(), imlib_image_get_height());
+#endif
+  imlib_free_image_and_decache();
+  return 0;
+
+ fail:
+  *pix = NoPixmapError;
+  return 1;
 }
 
-void    GetPixmaps(Display * d, struct window *win)
+static char pixmapDir[1024] = { '\0' };
+
+static void GetPixmapDir()
 {
-  register int i, j;
-  char    buf[1024], pixmapDir[1024];
   char   *pd;
-  int     missing;
 
-  mylocal = win;
-  W_Display = d;
+  if (pixmapDir[0] != '\0') return;
 
   pd = getdefault("pixmapDir");
   if (pd != (char *) NULL)
-    strcpy(pixmapDir, pd);
+    strncpy(pixmapDir, pd, 1024);
   else
     strcpy(pixmapDir, "/usr/share/pixmaps/netrek-client-cow");
 
-  if ((strcmpi(pixmapDir, "None") == 0) || (pixMissing & NO_PIXMAPS))
-    {
-      pixMissing = NO_IND_PIX | NO_FED_PIX | NO_ROM_PIX |
+  if ((strcmpi(pixmapDir, "None") == 0) || (pixMissing & NO_PIXMAPS)) {
+    pixMissing = NO_IND_PIX | NO_FED_PIX | NO_ROM_PIX |
+      NO_KLI_PIX | NO_ORI_PIX | NO_WEP_PIX | NO_EXP_PIX |
+      NO_CLK_PIX | NO_MAP_PIX | NO_BG_PIX | NO_PIXMAPS;
+    fprintf(stderr, "pixmaps turned off\n");
+  } else {
+    struct stat buf;
+
+    if ((stat(pixmapDir, &buf)) || (!(S_ISDIR(buf.st_mode)))) {
+      /* if .xtrekrc is wrong, and package default wrong, try default dir */
+      if ((stat("pixmaps", &buf)) || (!(S_ISDIR(buf.st_mode)))) {
+	pixMissing = NO_IND_PIX | NO_FED_PIX | NO_ROM_PIX |
 	  NO_KLI_PIX | NO_ORI_PIX | NO_WEP_PIX | NO_EXP_PIX |
 	  NO_CLK_PIX | NO_MAP_PIX | NO_BG_PIX | NO_PIXMAPS;
-      fprintf(stderr, "pixmaps turned off\n");
+	fprintf(stderr, "pixmaps not here\n");
+      } else {
+	strcpy(pixmapDir, "pixmaps");
+      }
     }
-  else
-    {
-      struct stat buf;
+  }
+}
 
-      if ((stat(pixmapDir, &buf)) || (!(S_ISDIR(buf.st_mode))))
-	{
-	  pixMissing = NO_IND_PIX | NO_FED_PIX | NO_ROM_PIX |
-	      NO_KLI_PIX | NO_ORI_PIX | NO_WEP_PIX | NO_EXP_PIX |
-	      NO_CLK_PIX | NO_MAP_PIX | NO_BG_PIX | NO_PIXMAPS;
-	  fprintf(stderr, "pixmaps not here\n");
-	}
-    }
+void    GetPixmaps(Display * d, struct window *win, W_Window t, W_Window g)
+{
+  register int i, j;
+  char    buf[1024];
+  int     missing;
 
-  pixmap_gc = XCreateGC(W_Display, win->window, 0, NULL);
+  Drawable tactical = W_Void2Window(t)->window;
+  Drawable galactic = W_Void2Window(g)->window;
+  W_Display = d;
+
+  GetPixmapDir();
 
   for (i = 0; i < NUMTEAM + 1; i++)
     {
@@ -260,7 +256,7 @@ void    GetPixmaps(Display * d, struct window *win)
       for (j = 0; j < NUM_TYPES; j++)
 	{
 	  sprintf(buf, "%s/%s/%s", pixmapDir, teamnames[i], shipfiles[j]);
-	  missing += ReadFileToSprite(buf, &shipImg[i][j], &w);
+	  missing += ReadFileToSprite(buf, &shipImg[i][j], tactical);
 	}
       if (missing == NUM_TYPES)
 	{
@@ -276,10 +272,10 @@ void    GetPixmaps(Display * d, struct window *win)
       for (j = 0; j < 2; j++)
 	{
 	  sprintf(buf, "%s/%s/%s", pixmapDir, teamnames[i], torpfiles[j]);
-	  missing += ReadFileToSprite(buf, &torpImg[i][j], &w);
+	  missing += ReadFileToSprite(buf, &torpImg[i][j], tactical);
 
 	  sprintf(buf, "%s/%s/%s", pixmapDir, teamnames[i], plasmafiles[j]);
-	  missing += ReadFileToSprite(buf, &plasmaImg[i][j], &w);
+	  missing += ReadFileToSprite(buf, &plasmaImg[i][j], tactical);
 	}
     }
   if (missing == (NUMTEAM + 1) * 4)
@@ -293,7 +289,7 @@ void    GetPixmaps(Display * d, struct window *win)
   for (i = 0; i < 2; i++)
     {
       sprintf(buf, "%s/Misc/%s", pixmapDir, explosionfiles[i]);
-      missing += ReadFileToSprite(buf, &explosionImg[i], &w);
+      missing += ReadFileToSprite(buf, &explosionImg[i], tactical);
     }
   if (missing == 2)
     {
@@ -309,14 +305,14 @@ void    GetPixmaps(Display * d, struct window *win)
       if (i == PL_PIX_AGRI && missing == 0)
 	{
 	  /* If the AGRI pixmap is missing, use the ROCK pixmap */
-	  if (ReadFileToSprite(buf, &mplanetImg[i], &mapw) != 0)
+	  if (ReadFileToSprite(buf, &mplanetImg[i], galactic) != 0)
 	    {
 	      sprintf(buf, "%s/Planets/%s", pixmapDir, mplanetfiles[i - 1]);
-	      ReadFileToSprite(buf, &mplanetImg[i], &mapw);
+	      ReadFileToSprite(buf, &mplanetImg[i], galactic);
 	    }
 	}
       else
-	missing += ReadFileToSprite(buf, &mplanetImg[i], &mapw);
+	missing += ReadFileToSprite(buf, &mplanetImg[i], galactic);
     }
   if (missing)
     {
@@ -326,7 +322,7 @@ void    GetPixmaps(Display * d, struct window *win)
     }
 
   sprintf(buf, "%s/Misc/%s", pixmapDir, cloakfile);
-  if (ReadFileToSprite(buf, &cloakImg, &w))
+  if (ReadFileToSprite(buf, &cloakImg, tactical))
     {
       pixMissing |= NO_CLK_PIX;
       if (!(pixMissing & NO_PIXMAPS))
@@ -337,7 +333,7 @@ void    GetPixmaps(Display * d, struct window *win)
   for (i = 0; i < NUM_BG_IMGS; i++)
     {
       sprintf(buf, "%s/Misc/%s", pixmapDir, bgfiles[i]);
-      missing += ReadFileToTile(buf, &backPix[i]);
+      missing += ReadFileToTile(buf, &backPix[i], galactic);
     }
   if (missing == NUM_BG_IMGS)
     {
@@ -356,7 +352,6 @@ void    GetPixmaps(Display * d, struct window *win)
 int     W_DrawSprite(void *in, int x, int y, int winside)
 {
   const int view = SCALE * winside / 2;
-  struct window *win;
   struct S_Object *sprite = (struct S_Object *) in;
   int     dx, dy;
 
@@ -366,8 +361,6 @@ int     W_DrawSprite(void *in, int x, int y, int winside)
   if (x > view || x < -view || y > view || y < -view)
     return 0;
 
-  win = W_Void2Window(*(sprite->parent));
-
   dx = x - (sprite->width) / 2;
   dy = y - (sprite->height) / 2;
 
@@ -384,7 +377,7 @@ int     W_DrawSprite(void *in, int x, int y, int winside)
 		     dx, dy - sprite->view * (sprite->height));
     }
 
-  XCopyArea(W_Display, sprite->image, win->window, sprite->gc,
+  XCopyArea(W_Display, sprite->image, sprite->drawable, sprite->gc,
 	    0, (sprite->view) * (sprite->height),
 	    sprite->width, sprite->height,
 	    dx, dy);
@@ -392,6 +385,32 @@ int     W_DrawSprite(void *in, int x, int y, int winside)
   return (sprite->width);
 }
 
+void W_DrawSpriteAbsolute(void *in, int x, int y)
+{
+  struct S_Object *sprite = (struct S_Object *) in;
+
+  if (sprite == NULL)
+    return;
+
+  XSetClipMask(W_Display, sprite->gc, sprite->shape);
+  XSetClipOrigin(W_Display, sprite->gc, x, y);
+  XCopyArea(W_Display, sprite->image, sprite->drawable, sprite->gc,
+            0, 0, sprite->width, sprite->height, x, y);
+}
+
+void W_ClearSpriteAbsolute(void *in, int x, int y)
+{
+  struct S_Object *sprite = (struct S_Object *) in;
+
+  if (sprite == NULL)
+    return;
+
+  XSetClipMask(W_Display, sprite->gc, sprite->shape);
+  XSetClipOrigin(W_Display, sprite->gc, x, y);
+  XClearArea(W_Display, sprite->drawable,
+             x, y, sprite->width, sprite->height, False);
+}
+
 void   *S_Ship(int playerno)
 {
   struct S_Object *sprite;
@@ -404,8 +423,8 @@ void   *S_Ship(int playerno)
   sprite = &(shipImg[remap[this->p_team]][this->p_ship.s_type]);
   sprite->cloak = 0;
 
-  /* The following generalizes "rosette(x)" for an arbitrary number of views
-   * * in the xpm    */
+  /* The following generalizes "rosette(x)" for an arbitrary number of
+     views in the image */
 
   sprite->view = ((((this->p_dir) + 128 / (sprite->nviews))
 		   / (256 / (sprite->nviews))) % (sprite->nviews));
@@ -424,7 +443,7 @@ void   *S_Ship(int playerno)
     }
   else if (this->p_status == PEXPLODE)
     {
-      int     i = this->p_explode;
+      int     i = this->p_explode * 5 / server_ups;
 
       if (pixFlags & NO_EXP_PIX)
 	return ((void *) NULL);
@@ -602,8 +621,8 @@ void   *S_Torp(int torpno)
     {
       sprite = &torpImg[remap[players[this->t_owner].p_team]][1];
       this->t_fuse--;
-      numdetframes = NUMDETFRAMES * server_ups / 10;
-      frame = this->t_fuse * 10 / server_ups;
+      numdetframes = NUMDETFRAMES * server_ups / 5;
+      frame = this->t_fuse * 5 / server_ups;
       if (this->t_fuse <= 0)
 	{
 	  this->t_status = PTFREE;
@@ -622,6 +641,7 @@ void   *S_Torp(int torpno)
     {
       sprite = &torpImg[remap[players[this->t_owner].p_team]][0];
       sprite->view = ++sprite->view % sprite->nviews;
+      // FIXME: torps rotate faster with higher client update rates
     }
 
   if ((sprite->image == NoPixmapError) || (pixFlags & NO_WEP_PIX))
@@ -691,4 +711,140 @@ void    W_LocalBgd(int which)
 
   W_ClearWindow(w);
 }
-#endif /* HAVE_XPM */
+
+void    W_SetBackground(W_Window w, int which)
+{
+  struct window *win = W_Void2Window(w);
+
+  if ((backPix[which] == NoPixmapError) || (pixFlags & NO_BG_PIX))
+    W_UnTileWindow(w);
+  else
+    XSetWindowBackgroundPixmap(W_Display, win->window, backPix[which]);
+
+  W_ClearWindow(w);
+}
+
+void *W_SetBackgroundImage(W_Window w, char *name)
+{
+  Drawable drawable = W_Void2Window(w)->window;
+  struct S_Object *sprite = calloc(1, sizeof(struct S_Object));
+  char *path;
+
+  if (sprite == NULL) return NULL;
+
+  GetPixmapDir();
+  path = malloc(strlen(pixmapDir) + strlen(name) + 2);
+  if (path == NULL) return NULL;
+
+  sprintf(path, "%s/%s", pixmapDir, name);
+
+  if (ReadFileToSprite(path, sprite, drawable)) {
+    free(path);
+    return NULL;
+  }
+  free(path);
+
+  XSetWindowBackgroundPixmap(W_Display, drawable, sprite->image);
+  W_ClearWindow(w);
+  return (void *) sprite;
+}
+
+static struct S_Object *ss = NULL;
+static int ss_size = 0;
+static int ss_next = 0;
+static int ss_show = 0;
+
+static void ss_init(W_Window w)
+{
+  char *path, *argv[2];
+  FTS *fts;
+  FTSENT *ent;
+  Drawable drawable = W_Void2Window(w)->window;
+
+  if (ss != NULL) return;
+
+  GetPixmapDir();
+
+  path = malloc(strlen(pixmapDir) + 4);
+  sprintf(path, "%s/ss", pixmapDir);
+  argv[0] = path;
+  argv[1] = NULL;
+
+  ss_size = 50;
+  ss = (struct S_Object *) malloc(ss_size * sizeof(struct S_Object));
+  ss_next = 0;
+
+  fts = fts_open(argv, FTS_LOGICAL, NULL);
+  while ((ent = fts_read(fts))) {
+    if (ent->fts_info != FTS_F) continue;
+    memset(&ss[ss_next], 0, sizeof(struct S_Object));
+    if (ReadFileToSprite(ent->fts_path, &ss[ss_next], drawable)) continue;
+    ss_next++;
+    if (ss_next >= ss_size) {
+      ss_size += 10;
+      ss = realloc(ss, ss_size * sizeof(struct S_Object));
+    }
+  }
+  fts_close(fts);
+  free(path);
+}
+
+void    W_NextScreenShot(W_Window w, int x, int y)
+{
+  ss_init(w);
+  if (ss_next == 0) return;
+  W_ClearSpriteAbsolute(&ss[ss_show], x, y);
+  ss_show++;
+  if (ss_show >= ss_next) ss_show=0;
+  W_DrawSpriteAbsolute(&ss[ss_show], x, y);
+}
+
+void    W_DrawScreenShot(W_Window w, int x, int y)
+{
+  ss_init(w);
+  if (ss_next == 0) return;
+  W_DrawSpriteAbsolute(&ss[ss_show], x, y);
+}
+
+void    *W_ReadImage(W_Window w, char *name)
+{
+  Drawable drawable = W_Void2Window(w)->window;
+  struct S_Object *sprite = calloc(1, sizeof(struct S_Object));
+  char *path;
+
+  if (sprite == NULL) return NULL;
+
+  GetPixmapDir();
+  path = malloc(strlen(pixmapDir) + strlen(name) + 2);
+  if (path == NULL) return NULL;
+
+  sprintf(path, "%s/%s", pixmapDir, name);
+
+  if (ReadFileToSprite(path, sprite, drawable)) {
+    free(path);
+    return NULL;
+  }
+  free(path);
+  return (void *) sprite;
+}
+
+void    W_DrawImage(int x, int y, void *sprite_v)
+{
+  struct S_Object *sprite = (struct S_Object *) sprite_v;
+
+  if (sprite == NULL) return;
+
+  W_DrawSpriteAbsolute(sprite, x, y);
+}
+
+void    W_DropImage(void *sprite_v)
+{
+  struct S_Object *sprite = (struct S_Object *) sprite_v;
+
+  if (sprite == NULL) return;
+
+  XFreeGC(W_Display, sprite->gc);
+  XFreePixmap(W_Display, sprite->image);
+  XFreePixmap(W_Display, sprite->shape);
+  free(sprite);
+}
