@@ -1,5 +1,5 @@
 /* x11window.c
- * 
+ *
  * Kevin P. Smith  6/11/89 Much modified by Jerry Frain and Joe Young
  *
  */
@@ -204,7 +204,7 @@ static XWMHints wm_hint =
 #endif
 
 static W_Event W_myevent;
-static int W_isEvent = 0;
+static int W_isEvent = 0; /* an event is being held here for the caller */
 
 struct fontInfo
   {
@@ -440,10 +440,6 @@ void
   GetFonts();
   GetColors();
 
-#ifdef HAVE_XPM
-  GetPixmaps(W_Display, &myroot);
-#endif
-
   /* display scroll thumb */
   scrollbar = booleanDefault("ScrollBar", scrollbar);
   scroll_lines = intDefault("ScrollSaveLines", scroll_lines);
@@ -452,7 +448,11 @@ void
 #ifdef BEEPLITE
   init_tts();
 #endif
-  W_FullScreenInitialise();
+}
+
+void W_GetPixmaps(W_Window t, W_Window g)
+{
+  GetPixmaps(W_Display, &myroot, t, g);
 }
 
 /* Make sure the font will work, ie: that it fits in the 6x10 character cell
@@ -941,8 +941,6 @@ W_MakeWindow(char *name, int x, int y, int width, int height, W_Window parent, i
 #endif
 
   gcheck_result = checkGeometry(name, &x, &y, &width, &height);
-  /* printf("%s.geometry: %d %d %d %d %d\n", name, x, y, width, height, *
-   * gcheck_result); */
   checkParent(name, &parent);
   wparent = W_Void2Window(parent)->window;
   attrs.border_pixel = colortable[color].pixelValue;
@@ -955,10 +953,6 @@ W_MakeWindow(char *name, int x, int y, int width, int height, W_Window parent, i
 #ifdef MOTION_MOUSE
   attrs.event_mask |= ButtonMotionMask;
 #endif
-  //  if (!strcmp(name, "netrek")) {
-  //    attrs.event_mask |= StructureNotifyMask;
-  //  }
-
 
   if (strcmp(name, "netrek_icon") == 0)		 /* icon should not select *
 						  * for input */
@@ -968,8 +962,7 @@ W_MakeWindow(char *name, int x, int y, int width, int height, W_Window parent, i
   newwin = newWindow(
 	      XCreateWindow(W_Display, wparent, x, y, width, height, border,
 			    CopyFromParent, InputOutput, CopyFromParent,
-			    CWBackPixel | CWEventMask |
-			    CWBorderPixel,
+			    CWBackPixel | CWEventMask | CWBorderPixel,
 			    &attrs),
 		      WIN_GRAPH);
 
@@ -1024,10 +1017,6 @@ W_MakeWindow(char *name, int x, int y, int width, int height, W_Window parent, i
   class_hint.res_name = name;
   XSetClassHint(W_Display, newwin->window, &class_hint);
   XSetWMHints(W_Display, newwin->window, &wm_hint);
-
-  if (wparent == W_Root && baseWin != NULL && strcmp(name, "wait") != 0
-      && strcmp(name, "waitmotd") != 0)
-    XSetTransientForHint(W_Display, newwin->window, W_Void2Window(baseWin)->window);
   newwin->name = strdup(name);
   newwin->width = width;
   newwin->height = height;
@@ -2013,8 +2002,6 @@ W_MakeTextWindow(char *name, int x, int y, int width, int height, W_Window paren
   XSetWMNormalHints(W_Display, newwin->window, sz_hints);
   XSetClassHint(W_Display, newwin->window, &class_hint);
   XSetWMHints(W_Display, newwin->window, &wm_hint);
-  if (wparent == W_Root && baseWin != NULL)
-    XSetTransientForHint(W_Display, newwin->window, W_Void2Window(baseWin)->window);
   newwin->name = strdup(name);
   newwin->width = width;
   newwin->height = height;
@@ -2148,9 +2135,6 @@ int     border;
   XFree((void *) sz_hints);
   XSetClassHint(W_Display, newwin->window, &class_hint);
   XSetWMHints(W_Display, newwin->window, &wm_hint);
-  if (wparent == W_Root && baseWin != NULL)
-    XSetTransientForHint(W_Display, newwin->window, W_Void2Window(baseWin)->window);
-
   newwin->name = strdup(name);
   sw = (struct scrollingWindow *) malloc(sizeof(struct scrollingWindow));
 
@@ -2427,8 +2411,6 @@ W_MakeMenu(char *name, int x, int y, int width, int height, W_Window parent, int
   class_hint.res_name = name;
   XSetClassHint(W_Display, newwin->window, &class_hint);
   XSetWMHints(W_Display, newwin->window, &wm_hint);
-  if (wparent == W_Root && baseWin != NULL)
-    XSetTransientForHint(W_Display, newwin->window, W_Void2Window(baseWin)->window);
   XStoreName(W_Display, newwin->window, name);
   newwin->name = strdup(name);
   items = (struct menuItem *) malloc(height * sizeof(struct menuItem));
@@ -3026,15 +3008,6 @@ void
   free((char *) win);
 }
 
-/* NOT USED */
-void
-        W_SetTransientForHint(W_Window w, W_Window pw)
-{
-  XSetTransientForHint(W_Display, W_Void2Window(w)->window,
-		       W_Void2Window(pw)->window);
-}
-
-
 void deleteWindow(struct window *window)
 {
   struct windowlist **rm;
@@ -3419,17 +3392,17 @@ void
 
 
 {
-  Window  win = W_Void2Window(window)->window;
+  struct window *w = W_Void2Window(window);
   XSizeHints *sz_hints;
 
   sz_hints = XAllocSizeHints();
-  sz_hints->min_width = (unsigned int) neww;
-  sz_hints->max_width = (unsigned int) neww;
-  sz_hints->min_height = (unsigned int) newh;
-  sz_hints->max_height = (unsigned int) newh;
+  sz_hints->min_width = neww;
+  sz_hints->max_width = neww;
+  sz_hints->min_height = newh;
+  sz_hints->max_height = newh;
   sz_hints->flags = PMinSize | PMaxSize;
-  XSetWMNormalHints(W_Display, win, sz_hints);
-  XResizeWindow(W_Display, win, (unsigned int) neww, (unsigned int) newh);
+  XSetWMNormalHints(W_Display, w->window, sz_hints);
+  XResizeWindow(W_Display, w->window, neww, newh);
 }
 
 void
@@ -3470,15 +3443,7 @@ W_ResizeMenu(W_Window window, int neww, int newh) /* TSH 2/93 */
 }
 
 void
-        W_ResizeTextWindow(W_Window window, int neww, int newh)		/* TSH 
-									 * 
-									 * *
-									 * 2/93 
-									 * 
-									 */
-
-
-
+        W_ResizeTextWindow(W_Window window, int neww, int newh) /* TSH 2/93 */
 {
   W_ResizeWindow(window, neww * W_Textwidth + WIN_EDGE * 2,
 		 newh * W_Textheight + WIN_EDGE * 2);
@@ -3676,7 +3641,6 @@ void
 }
 #endif
 
-#ifdef HAVE_XPM
 void    W_Halo(int x, int y, W_Color color)
 {
   struct window *win = W_Void2Window(mapw);
@@ -3690,7 +3654,6 @@ void    W_Halo(int x, int y, W_Color color)
 	       mplanet_width, mplanet_height, 0, 23040);
     }
 }
-#endif /* HAVE_XPM */
 
 void W_CameraSnap(W_Window window)
 {
@@ -3799,7 +3762,6 @@ static void view_port_warp(W_Window window)
   Window child;
   XTranslateCoordinates(W_Display, win->window, W_Root, 0, 0, &tx, &ty, &child);
   XF86VidModeSetViewPort(W_Display, W_Screen, tx, ty);
-  XMoveResizeWindow(W_Display, win->window, 0, 0, 1024, 768);
   XMapRaised(W_Display, win->window);
   XRaiseWindow(W_Display, win->window);
 }
@@ -3819,6 +3781,7 @@ static void pointer_grab_on(W_Window window)
                  win->window, None, CurrentTime);
   XGrabKeyboard(W_Display, win->window, True, GrabModeAsync,
                 GrabModeAsync, CurrentTime);
+  XFlush(W_Display);
 }
 
 static void pointer_grab_off(W_Window window)
@@ -3832,7 +3795,7 @@ static void kde_fullscreen_on(W_Window window) {
   Atom WM_HINTS;
   WM_HINTS = XInternAtom(W_Display, "_NET_WM_STATE", True);
   if (WM_HINTS != None) {
-    Atom p[1]; 
+    Atom p[1];
     p[0] = XInternAtom(W_Display, "_NET_WM_STATE_FULLSCREEN", True);
     XChangeProperty(W_Display, win->window, WM_HINTS, XA_ATOM, 32,
                     PropModeReplace, (unsigned char *)p, 1);
@@ -3852,13 +3815,15 @@ static void kde_fullscreen_off(W_Window window) {
 
 void W_FullScreenOn(W_Window window)
 {
+  struct window *win = W_Void2Window(window);
 #ifdef FULLSCREEN
 #if DEBUG > 0
   fprintf(stderr, "W_FullScreenOn\n");
 #endif
+  XResizeWindow(W_Display, win->window, 1024, 768);
+  pointer_grab_on(window);
   video_mode_on();
   view_port_warp(window);
-  pointer_grab_on(window);
   kde_fullscreen_on(window);
 #endif
 }
@@ -3883,7 +3848,6 @@ void W_FullScreenInitialise() {
   full_screen_enabled = 0;
   full_screen_default = 0;
   if (booleanDefault("FullScreen", 0)) {
-    // FIXME: when this is on in .xtrekrc normal resolution may not be restored
     full_screen_default++;
     if (video_mode_initialise())
       full_screen_enabled++;
